Package: SPsimSeq
Title: Semi-parametric simulation tool for bulk and single-cell RNA
        sequencing data
Version: 1.23.0
Authors@R: 
    c(person(given = "Alemu Takele",
           family = "Assefa",
           role = c("aut"),
           email = "alemutak@hotmail.com"),
    person(given = "Olivier",
           family = "Thas",
           role = c("ths"),
           email = "Olivier.Thas@ugent.be"),
    person(given = "Joris",
           family = "Meys",
           role = c("cre"),
           email = "Joris.Meys@ugent.be"),
    person(given = "Stijn",
           family = "Hawinkel",
           role = c("aut"),
           email = "stijnhawinkel@hotmail.com"))
Description: SPsimSeq uses a specially designed exponential family for
        density estimation to constructs the distribution of gene
        expression levels from a given real RNA sequencing data
        (single-cell or bulk), and subsequently simulates a new dataset
        from the estimated marginal distributions using
        Gaussian-copulas to retain the dependence between genes. It
        allows simulation of multiple groups and batches with any
        required sample size and library size.
License: GPL-2
Encoding: UTF-8
LazyData: true
URL: https://github.com/CenterForStatistics-UGent/SPsimSeq
Imports: stats, methods, SingleCellExperiment, fitdistrplus, graphics,
        edgeR, Hmisc, WGCNA, limma, mvtnorm, phyloseq, utils
biocViews: GeneExpression, RNASeq, SingleCell, Sequencing, DNASeq
RoxygenNote: 7.1.0
Suggests: knitr, rmarkdown, LSD, testthat, BiocStyle
VignetteBuilder: knitr
Depends: R (>= 4.0)
Config/pak/sysreqs: cmake libglpk-dev make libicu-dev libuv1-dev
        libxml2-dev zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:52:57 UTC
RemoteUrl: https://github.com/bioc/SPsimSeq
RemoteRef: HEAD
RemoteSha: 72d88fe83edbdf2bcbcc993a08bdec977ca35ca3
NeedsCompilation: no
Packaged: 2026-05-30 09:48:31 UTC; root
Author: Alemu Takele Assefa [aut],
  Olivier Thas [ths],
  Joris Meys [cre],
  Stijn Hawinkel [aut]
Maintainer: Joris Meys <Joris.Meys@ugent.be>
Built: R 4.6.0; ; 2026-05-30 14:48:01 UTC; unix

# Generated by roxygen2: do not edit by hand

export(SPsimSeq)
export(configExperiment)
export(evaluateDensities)
import(methods)
importFrom(Hmisc,cut2)
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,counts)
importFrom(WGCNA,cor)
importFrom(edgeR,calcNormFactors)
importFrom(fitdistrplus,fitdist)
importFrom(graphics,hist)
importFrom(limma,voom)
importFrom(mvtnorm,rmvnorm)
importFrom(phyloseq,otu_table)
importFrom(phyloseq,t)
importFrom(phyloseq,taxa_are_rows)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,glm)
importFrom(stats,glm.fit)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,pnorm)
importFrom(stats,poisson)
importFrom(stats,rbinom)
importFrom(stats,rlnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(utils,combn)

## ----githubInstall, eval=FALSE------------------------------------------------
# ## Install SPsimSeq
# library(devtools)
# install_github("CenterForStatistics-UGent/SPsimSeq")

## ----BiocInstall, eval=FALSE--------------------------------------------------
# ## Install SPsimSeq
# library(BiocManager)
# BiocManager::install("SPsimSeq")

## ----loadSPsimSeq-------------------------------------------------------------
# load package
library(SPsimSeq) 

## ----loadData, eval=TRUE------------------------------------------------------
 # load the Zhang bulk RNA-seq data (availabl with the package) 
 data("zhang.data.sub") 
  # filter genes with sufficient expression (important step to avoid bugs) 
 zhang.counts <- zhang.data.sub$counts 
 MYCN.status  <- zhang.data.sub$MYCN.status #The grouping variable

## ----simulateData-------------------------------------------------------------
 set.seed(6452) #Set seed for reproducibility
 # simulate data
 sim.data.bulk <- SPsimSeq(n.sim = 1, s.data = zhang.counts,
                          group = MYCN.status, n.genes = 3000, batch.config = 1,
                          group.config = c(0.5, 0.5), tot.samples = ncol(zhang.counts), 
                          pDE = 0.1, lfc.thrld = 0.5, result.format = "list", return.details = TRUE)

## ----dataExploration----------------------------------------------------------
 sim.data.bulk1 <- sim.data.bulk$sim.data.list[[1]]
 head(sim.data.bulk1$counts[, seq_len(5)])  # count data
 head(sim.data.bulk1$colData)        # sample info
 head(sim.data.bulk1$rowData)        # gene info

## ----evaluateDensities--------------------------------------------------------
geneDens = evaluateDensities(sim.data.bulk, newData = rownames(zhang.counts)[1])
#This returns for every sample, the midpoints (mids) and associated densities (gy)

## ----comparison, warning=FALSE, fig.width=8, fig.height=4---------------------
# compare the distributions of the mean expressions, variability, 
# and fraction of zero counts per gene
library(LSD) # for generating heatmap plots
# normalize counts for comparison  
Y0.log.cpm <- log2(edgeR::cpm(zhang.counts)+1)
Y1.log.cpm <- log2(edgeR::cpm(sim.data.bulk1$counts)+1)
Y0.log.cpm <- Y0.log.cpm[rowMeans(Y0.log.cpm>0)>=0.1, ]
Y1.log.cpm <- Y1.log.cpm[rowMeans(Y1.log.cpm>0)>=0.1, ]
rowVars <- function(X){apply(X, 1, var, na.rm=TRUE)}
rowCVs <- function(X){apply(X, 1, function(x) sd(x, na.rm=TRUE)/mean(x, na.rm=TRUE))}
par(mfrow=c(1, 3))
boxplot(list(real.data=log(colSums(zhang.counts)), 
             simulated.data=log(sim.data.bulk1$colData$sim.Lib.Size)), 
        main="library size") 
boxplot(list(real.data=rowMeans(Y0.log.cpm), 
             simulated.data=rowMeans(Y1.log.cpm)), 
        main="mean expression of genes") 
boxplot(list(real.data=rowVars(Y0.log.cpm), 
             simulated.data=rowVars(Y1.log.cpm)), 
        main="variance of gene expressions") 

## ----meanvariancetrend--------------------------------------------------------
# compare the relationship between the mean and variability
par(mfrow=c(1,3), mar=c(4,4,4,1))
heatscatter(rowMeans(Y0.log.cpm), rowCVs(Y0.log.cpm), ylim=c(0, 6), xlim=c(0, 16),
            colpal="bl2gr2rd", main="real data", xlab="mean log2-CPM", 
            ylab="coefficients of variation", cexplot=0.5, alpha = 60, cex.lab=1.25)
heatscatter(rowMeans(Y1.log.cpm), rowCVs(Y1.log.cpm), ylim=c(0, 6), xlim=c(0, 16),
     main="SPsimSeq", xlab="mean log2-CPM", ylab="coefficients of variation", 
     cexplot=0.5, alpha = 60, colpal="bl2gr2rd", cex.lab=1.25)

n.gride <- 1000
min.g   <- seq(0, 20, length.out = n.gride+1)[-n.gride]
max.g   <- seq(0, 20, length.out = n.gride+1)[-1] 
mid.g   <- (min.g+max.g)/2
f.real  <- vapply(seq_len(n.gride), FUN.VALUE = double(1), function(r){
  x <- Y0.log.cpm[rowMeans(Y0.log.cpm)<=max.g[r] & rowMeans(Y0.log.cpm)>min.g[r],]
  y <- ifelse(!is.null(dim(x)), mean(rowCVs(x)), mean(sd(x)/mean(x))) 
  y
})
f.SPsim <- vapply(seq_len(n.gride), FUN.VALUE = double(1), function(r){
  x <- Y1.log.cpm[rowMeans(Y1.log.cpm)<=max.g[r] & rowMeans(Y1.log.cpm)>min.g[r],]
  y <- ifelse(!is.null(dim(x)), mean(rowCVs(x)), mean(sd(x)/mean(x))) 
  y
})

sm1 <- loess(I(f.SPsim-f.real)~mid.g) 
plot(mid.g, f.SPsim-f.real, xlim=c(0, 14), col="lightskyblue", pch=20, cex.lab=1.25,
     cex.main=1.4, main="SPsimSeq - real data", ylab="difference", xlab="mean log2-CPM")
lines(mid.g,predict(sm1, newdata = mid.g), col="blue", lwd=3) 

## ----correlation--------------------------------------------------------------
# compare the correlation between genes and samples 
cor.mat.Y0 <- cor(t(Y0.log.cpm))
cor.mat.Y1 <- cor(t(Y1.log.cpm)) 
cor.vec.Y0 <- cor.mat.Y0[upper.tri(cor.mat.Y0)]
cor.vec.Y1 <- cor.mat.Y1[upper.tri(cor.mat.Y1)] 
par(mfrow=c(1,3), mar=c(4,4,3.5,1))
hist(cor.vec.Y0, nclass = 30, probability = TRUE, 
     border="gray", col="steelblue1", main="real data", xlab="Genewise correlations", 
     ylim=c(0, 3.5), xlim=c(-1, 1), cex.lab=1.25)
hist(cor.vec.Y1, nclass = 30, probability = TRUE, border="gray",
     col="steelblue1",  main="SPsimSeq", xlab="Genewise correlations",
     ylim=c(0, 3.5), xlim=c(-1, 1), cex.lab=1.25)
plot(seq(-1, 1, 0.1), seq(-1, 1, 0.1), type="n", xlab="quantile (real data)", 
     ylab="quantile (simulated data)",  main="correlation quantile-quantile plot")
abline(0, 1, col="gray")
points(quantile(cor.vec.Y0, seq(0, 1, 0.001)), quantile(cor.vec.Y1, seq(0, 1, 0.001)), 
       col="blue", pch=20, cex=1.5, cex.lab=1.25)  

## ----scRNA, eval=TRUE, warning=FALSE, fig.width=8, fig.height=4---------------
library(SingleCellExperiment)
# load the NGP nutlin data (availabl with the package, processed with SMARTer/C1 protocol, and contains read-counts)
  data("scNGP.data")
  set.seed(654321)
 # simulate data (we simulate here only a single data, n.sim = 1)
 sim.data.sc <- SPsimSeq(n.sim = 1, s.data = scNGP.data,
                         group = scNGP.data$characteristics..treatment, 
                         n.genes = 4000, batch.config = 1,
                         group.config = c(0.5, 0.5), tot.samples = 100, 
                         pDE = 0.1, lfc.thrld = 0.5, model.zero.prob = TRUE,
                         result.format = "SCE")

## ----scRNAhead----------------------------------------------------------------
 sim.data.sc1 <- sim.data.sc[[1]]
 class(sim.data.sc1)
 head(counts(sim.data.sc1)[, seq_len(5)])
 colData(sim.data.sc1)
 rowData(sim.data.sc1)

## ----basicSCrna---------------------------------------------------------------
# normalize counts for comparison  
Y0.log.cpm  <- log2(edgeR::cpm(counts(scNGP.data))+1)
Y1.log.cpm  <- log2(edgeR::cpm(counts(sim.data.sc1))+1)
Y0.log.cpm  <- Y0.log.cpm[rowMeans(Y0.log.cpm>0)>=0.1, ]
Y1.log.cpm  <- Y1.log.cpm[rowMeans(Y1.log.cpm>0)>=0.1, ]
rowVars <- function(X){apply(X, 1, var, na.rm=TRUE)}
rowCVs <- function(X){apply(X, 1, function(x) sd(x, na.rm=TRUE)/mean(x, na.rm=TRUE))}
rowZeroFrac <- function(X){apply(X, 1, function(x) mean(x==0, na.rm=TRUE))}
par(mfrow=c(1, 3))
boxplot(list(real.data=colSums(counts(scNGP.data)), 
             simulated.data=colData(sim.data.sc1)$sim.Lib.Size), 
        main="library size") 
boxplot(list(real.data=rowMeans(Y0.log.cpm), 
             simulated.data=rowMeans(Y1.log.cpm)), 
        main="mean expression of genes") 
boxplot(list(real.data=rowVars(Y0.log.cpm), 
             simulated.data=rowVars(Y1.log.cpm)), 
        main="variance of gene expressions")

## ----meanVarSCrna-------------------------------------------------------------
# compare the relationship between the mean and variability  
par(mfrow=c(1,3), mar=c(4,4,4,1))
heatscatter(rowMeans(Y0.log.cpm), rowCVs(Y0.log.cpm), ylim=c(0, 6), xlim=c(0, 16),
            colpal="bl2gr2rd", main="real data", xlab="mean log2-CPM", 
            ylab="coefficients of variation", cexplot=0.5, alpha = 60, cex.lab=1.25)
heatscatter(rowMeans(Y1.log.cpm), rowCVs(Y1.log.cpm), ylim=c(0, 6), xlim=c(0, 16),
     main="SPsimSeq", xlab="mean log2-CPM", ylab="coefficients of variation", 
     cexplot=0.5, alpha = 60, colpal="bl2gr2rd", cex.lab=1.25)
n.gride <- 1000
min.g   <- seq(0, 20, length.out = n.gride+1)[-n.gride]
max.g   <- seq(0, 20, length.out = n.gride+1)[-1] 
mid.g   <- (min.g+max.g)/2
f.real  <- vapply(seq_len(n.gride), FUN.VALUE = double(1), function(r){
  x <- Y0.log.cpm[rowMeans(Y0.log.cpm)<=max.g[r] & rowMeans(Y0.log.cpm)>min.g[r],]
  y <- ifelse(!is.null(dim(x)), mean(rowCVs(x)), mean(sd(x)/mean(x))) 
  y
})
f.SPsim <- vapply(seq_len(n.gride), FUN.VALUE = double(1), function(r){
  x <- Y1.log.cpm[rowMeans(Y1.log.cpm)<=max.g[r] & rowMeans(Y1.log.cpm)>min.g[r],]
  y <- ifelse(!is.null(dim(x)), mean(rowCVs(x)), mean(sd(x)/mean(x))) 
  y
})

sm1 <- loess(I(f.SPsim-f.real)~mid.g) 
plot(mid.g, f.SPsim-f.real, xlim=c(0, 14), col="lightskyblue", pch=20, cex.lab=1.25,
     cex.main=1.4, main="SPsimSeq - real data", ylab="difference", xlab="mean log2-CPM")
lines(mid.g,predict(sm1, newdata = mid.g), col="blue", lwd=3)

## ----scRNAmeanzeroes----------------------------------------------------------
# compare the relationship between the  mean and fraction of zeros
par(mfrow=c(1,3), mar=c(4,4,4,1))
heatscatter(rowMeans(Y0.log.cpm), rowZeroFrac(Y0.log.cpm), ylim=c(0, 1), 
            xlim=c(0, 16), colpal="bl2gr2rd", main="real data", xlab="mean log2-CPM", 
            ylab="fraction of zero counts", cexplot=0.5, alpha = 60, cex.lab=1.25)
heatscatter(rowMeans(Y1.log.cpm), rowZeroFrac(Y1.log.cpm), ylim=c(0, 1), 
            xlim=c(0, 16), main="SPsimSeq", xlab="mean log2-CPM", 
            ylab="fraction of zero counts",  cexplot=0.5, alpha = 60,
            colpal="bl2gr2rd", cex.lab=1.25)
n.gride <- 1000
min.g   <- seq(0, 20, length.out = n.gride+1)[-n.gride]
max.g   <- seq(0, 20, length.out = n.gride+1)[-1] 
mid.g   <- (min.g+max.g)/2
f.real  <- vapply(seq_len(n.gride), FUN.VALUE = double(1), function(r){
  x <- Y0.log.cpm[rowMeans(Y0.log.cpm)<=max.g[r] & rowMeans(Y0.log.cpm)>min.g[r],]
  y <- ifelse(!is.null(dim(x)), mean(rowZeroFrac(x)), mean(x==0)) 
  y
})
f.SPsim <- vapply(seq_len(n.gride), FUN.VALUE = double(1), function(r){
  x <- Y1.log.cpm[rowMeans(Y1.log.cpm)<=max.g[r] & rowMeans(Y1.log.cpm)>min.g[r],]
  y <- ifelse(!is.null(dim(x)), mean(rowZeroFrac(x)), mean(x==0)) 
  y
})
sm1 <- loess(I(f.SPsim-f.real)~mid.g) 
plot(mid.g, f.SPsim-f.real, xlim=c(0, 14), col="lightskyblue", pch=20, cex.lab=1.25,
     cex.main=1.4, main="SPsimSeq - real data", ylab="difference", xlab="mean log2-CPM")
lines(mid.g,predict(sm1, newdata = mid.g), col="blue", lwd=3)

## ----corSCrna-----------------------------------------------------------------
# compare the correlation between genes and samples
Y0.log.cpm2 <- Y0.log.cpm[rowMeans(Y0.log.cpm>0)>0.25, ]
Y1.log.cpm2 <- Y1.log.cpm[rowMeans(Y1.log.cpm>0)>0.25, ]

cor.mat.Y0 <- cor(t(Y0.log.cpm2))
cor.mat.Y1 <- cor(t(Y1.log.cpm2)) 

cor.vec.Y0 <- cor.mat.Y0[upper.tri(cor.mat.Y0)]
cor.vec.Y1 <- cor.mat.Y1[upper.tri(cor.mat.Y1)] 

par(mfrow=c(1,3), mar=c(4,4,3.5,1))
hist(cor.vec.Y0, nclass = 30, probability = TRUE, 
     border="gray", col="steelblue1", main="real data", xlab="pairwise correlation between genes", 
     ylim=c(0, 3.5), xlim=c(-1, 1), cex.lab=1.25)
hist(cor.vec.Y1, nclass = 30, probability = TRUE, border="gray",
     col="steelblue1",  main="SPsimSeq", xlab="pairwise correlation between genes",
     ylim=c(0, 3.5), xlim=c(-1, 1), cex.lab=1.25)

plot(seq(-1, 1, 0.1), seq(-1, 1, 0.1), type="n", xlab="quantile (real data)", 
     ylab="quantile (simulated data)",  main="correlation quantile-quantile plot")
abline(0, 1, col="gray")
points(quantile(cor.vec.Y0, seq(0, 1, 0.001)), quantile(cor.vec.Y1, seq(0, 1, 0.001)), 
       col="blue", pch=20, cex=1.5, cex.lab=1.25)  

## -----------------------------------------------------------------------------
sessionInfo()

#' Evaluate the densities in the estimated SPsimSeq object
#' @param SPobj The SPsimSeq object, with details retained
#' @param newData A character vector of gene names
#' @return a list of estimated densities, breaks and midpoints, one for every
#' gene in newData
#' @export
#' @examples
#' data("zhang.data.sub")
#' # filter genes with sufficient expression (important step to avoid bugs)
#' zhang.counts <- zhang.data.sub$counts
#' MYCN.status  <- zhang.data.sub$MYCN.status
#' # simulate data
#' sim.data.bulk <- SPsimSeq(n.sim = 1, s.data = zhang.counts,
#'                           group = MYCN.status, n.genes = 100, batch.config = 1,
#'                           group.config = c(0.5, 0.5), tot.samples = 20,
#'                           pDE = 0.1, lfc.thrld = 0.5, result.format = "list",
#'                           return.details = TRUE)
#' outDens = evaluateDensities(sim.data.bulk)
#' select.genes <- sample(names(outDens), 4)
#' select.sample = sample(
#' seq_along(sim.data.bulk$detailed.results$exprmt.design$sub.groups), 1)
#' par(mfrow=c(2, 2))
#' for(i in select.genes){
#'      plot(outDens[[i]][[select.sample]]$mids, outDens[[i]][[select.sample]]$gy, type = "l",
#'      xlab = "Outcome", ylab = "Density", main = paste("Gene", i))
#'   }
evaluateDensities <- function(SPobj, newData = names(SPobj$detailed.results$densList)){
  if(!"detailed.results" %in% names(SPobj)){
    stop("Estimated densities needed, try running SPsimSeq with
         return.details = TRUE")
  }
  if(!is.character(newData)){
    stop("Provide a character vector of genes")
  }
  dets = SPobj$detailed.results #details
  names(newData) = newData
  #Construct the appropriate densities
  lapply(newData, function(gene){
    constructDens(returnDens = TRUE,
                  DE.ind.ii = gene %in% dets$cand.DE.genes$nonnull.genes,
                  exprmt.design = dets$exprmt.design,
                  densList.ii = dets$densList[[gene]])
  })
}

#' Neuroblastoma bulk RNA-seq data retrieved from Zhang et (2015).
#'
#' The data contains 498 neuroblastoma tumors. In short, unstranded
#' poly(A)+ RNA sequencing was performed on the HiSeq 2000 instrument (Illumina).
#' Paired-end reads with a length of 100 nucleotides were obtained. To quantify
#' the full transcriptome, raw fastq files were processed with Kallisto v0.42.4
#' (index build with GRCh38-Ensembl v85). The pseudo-alignment tool Kallisto
#' was chosen above other quantification methods as it is performing equally
#' good but faster. For this study, a subset of 172 tumors (samples) with
#' high-risk disease were selected, forming two groups: the MYCN amplified
#' ($n_1$ = 91) and MYCN non-amplified ($n_2$ = 81) tumours. Sometimes we refer
#' this dataset to as the Zhang data or the Zhang neuroblastoma data. In this
#' package, a subset of 5000 genes (randomly selected) are made available for illustration
#' purpose only.
#'
#' @docType data
#'
#' @format A list object
#'
#' @usage data(zhang.data.sub)
#'
#' @keywords datasets
#'
#' @references
#' 1. Zhang W, Yu Y, Hertwig F, Thierry-Mieg J, Zhang W, Thierry-Mieg D, Wang J, Furlanello C, Devanarayan V, Cheng J, et al. Comparison of RNA-seq and microarray-based models for clinical endpoint prediction. Genome Biol. 2015;16(133) https://doi.org/10.1186/s13059-015-0694-1
#' 2. Assefa, A. T., De Paepe, K., Everaert, C., Mestdagh, P., Thas, O., & Vandesompele, J. (2018). Differential gene expression analysis tools exhibit substandard performance for long non-coding RNA-sequencing data. GENOME BIOLOGY, 19.
#' \describe{
#'   \item{counts}{gene counts}
#'   \item{group}{MYCN (0 for MYCN non-amplified and 1 for MYCN amplified)}
#' }
#' @source \url{GEO accession GSE49711}
#' @examples
#' data("zhang.data.sub")
#' str(zhang.data.sub)
"zhang.data.sub"

#' A function that generates the simulated data for a single gene
#'
#' @param cumDens cumulative density
#' @param sel.genes.ii selected gene
#' @param log.CPM.transform a boolean, is log-CPM transform required?
#' @param prior.count the prior count
#' @param LL the library sizes
#' @param copSam the generated copula
#' @param model.zero.prob a boolean, should the zeroes be modelled separately
#' @param fracZero.logit.list The zero model
#' @param const.mult a large constant for the CPM transform, normally 1e6
#' @param exprmt.design the experiment design
#'
#' @return Simulated cpm values
SPsimPerGene <- function(cumDens, exprmt.design, 
                         sel.genes.ii, log.CPM.transform, prior.count, LL,
                         copSam, model.zero.prob, fracZero.logit.list, 
                         const.mult){
    ## Match with copula to simulate data 
    Y.star <- matchCopula(cumDens = cumDens, exprmt.design = exprmt.design, 
                          copSam = copSam, sel.genes.ii = sel.genes.ii)
    ## Back tranform to counts if needed
    if(log.CPM.transform){
      Y.star = round(((exp(Y.star) - prior.count)*LL)/const.mult)
      Y.star[Y.star<0] = 0L
    }
    ## Add zeroes if needed
    if(model.zero.prob){
      logLL = log(LL)
      zeroIds = vapply(seq_along(Y.star), FUN.VALUE = logical(1), function(i){
        samZeroID(fracZero.logit.list[[exprmt.design$sub.batchs[[i]]]], 
                   logLL = logLL[i], gene = sel.genes.ii)
      })
      Y.star[zeroIds] = 0L
    }
   return(Y.star)
}

#' Check for data validity
#'
#' @param s.data,group,batch,group.config,batch.config,w,log.CPM.transform,prior.count,pDE,lib.size.params,llStat.thrld,result.format see ?SPsimSeq
#'
#' @return Throws errors where neede, otherwise returns invisible
checkInputValidity <- function(s.data, group, batch, group.config, batch.config,
                               w, log.CPM.transform, prior.count, pDE,
                              lib.size.params, llStat.thrld, result.format){
  # Check for class of missing values in the source data
  if(anyNA(group)){
    stop("The group indicator contains missing values!")
  }
  if(anyNA(batch)){
    stop("The batch indicator contains missing values!")
  }
  if(anyNA(s.data)){
    stop("The source count matrix contains missing values")
  } 
  # Check for group/batch coformation with count matrix
  if(!is.null(group)){
    if(length(group) != ncol(s.data)){
      stop("The length of the group variable is not conformable with the 
      number of columns in the source count matrix!")
    }
  }
  if(!is.null(batch)){
    if(length(group) != ncol(s.data)){
      stop("The length of the batch variable is not conformable with the 
      number of columns in the source count matrix!")
    }
  }
  # Check for the validity of the group/batch configuration
  if(sum(group.config)!=1 | sum(batch.config)!=1){
    stop("The group.cofig and batch.config must sum to 1!")
  }
  # Check for the validity of other inputs
  if(!is.null(group)){
    if(length(group.config) > length(unique(group))){
      stop("The number of groups to be simulated is larger than the number of 
      groups available in the source data!")
    } else if((length(group.config) < length(unique(group))) & length(group.config) !=1){
     stop("The number of groups to be simulated is less than the number of 
      groups available in the source data. Maybe subset your source data OR consider 
      adding a 0 in the group.config vector for the group that you do not want to simulate 
      from. Example group.config=c(0.5, 0.5, 0)!")
    } else if((length(unique(group))!=1) & length(group.config) ==1){
      stop("The source data has", length(unique(group)), "groups but you are
                      simulating only for one group (length(group.config)=1). Hence, data 
                      only from one of the groups will be used!")
    } 
  }
  if(length(batch.config) > length(unique(batch))){
      stop("The number of batchs to be simulated is larger than the number of batches 
      available in the source data!")
  }
  if(!is.null(lib.size.params) & (length(lib.size.params) != 2 | is.null(names(lib.size.params)))){
    stop("The log-normal parameters for the distribution of library sizes must be submitted in a named vector of size 2. 
             Example, lib.size.params = c(meanlog=10, sdlog=0.2). See also ?rlnorm()")
  }
  if(!is.null(w) && (w < 0 || w > 1)){
    stop("w should be NULL or any value between 0 and 1 excluding 0 and 1")
  }
  if(prior.count < 0){
    stop("Prior count must be strictly positive!")
  }
  if(llStat.thrld <0){
    stop("Likelihood ratio test statistic threshold should be non-negative")
  }
  if(!(result.format %in% c("SCE", "list"))){
    stop("Result should be either SCE or list!")
  }
  if(pDE>1 || pDE<0){
    stop("Fraction of differential features pDE should lie between 0 and 1!")
  }
  invisible()
}

#' Calculates height and mid points of a distribution
#' 
#' @param Y a vector of gene expression data for a particular gene (in log CPM)
#' @param w a numeric value between 0 and 1 or NULL refering the number of classes to be created
# for the outcome data (if NULL the algorithm in graphics::hist() function will be used)
# 
#' @return a list object contating class breaks, mid points and counts
#' @importFrom graphics hist
obtCount <- function(Y, w){
  h = hist(Y, plot = FALSE, 
           nclass = if(is.null(w)) NULL else nclass = round(w*length(Y))) 
  list(breaks = h$breaks, mids = h$mids, counts = h$counts) 
}

#' Predict zero probability using logistic rgression
#' @param cpm.data  log CPM matrix
#' @param logL log library size of the source data
#' @param zeroMat the matrix of zero indicators
#' @param n.mean.class a fraction of the number of genes
#' for the number of groups to be created for the mean log CPM of genes
#'
#' @return The coefficients of the estimated logistic regression
#' @importFrom stats glm.fit binomial model.matrix
#' @importFrom Hmisc cut2
zeroProbModel <- function(cpm.data, logL, zeroMat, n.mean.class){
  # Fit a logistic model for the probability of zeros, if there are enough of them
  mean.log.cpm <- rowMeans(cpm.data) #Means
  n.mean.class <- round(n.mean.class*nrow(cpm.data))
  mid.val <- as.numeric(as.character(cut2(mean.log.cpm, g = n.mean.class, 
                                          levels.mean = TRUE)))
  x1 <- rep(mid.val, times = length(logL))
  x2 <- rep(logL, each = length(mid.val))
  desMat = model.matrix(~x1*x2)
  fracZero.logit <- suppressWarnings(glm.fit(y = c(zeroMat), x = desMat, 
                                             family = binomial()))
  return(fracZero.logit$coef)
}

#' Match copulas to estimated SP distribution
#'
#' @param cumDens The cumulative densities evaluated
#' @param exprmt.design the design
#' @param copSam the sampled copula
#' @param sel.genes.ii the gene
#'
#' @return the outcome values as a vector
#' @importFrom stats runif
matchCopula = function(cumDens, exprmt.design, copSam, sel.genes.ii){
  vapply(seq_along(exprmt.design$sub.batchs), function(i){
    copula = copSam[sel.genes.ii,i]
    cd = cumDens[[i]]
    id = which.min(abs(cd$Gy-copula))
    runif(1, cd$breaks[id], cd$breaks[id+1])
  }, FUN.VALUE = numeric(1))
}

#' SPsimSeq package
#'
#' SPsimSeq uses a specially designed exponential family for density estimation 
#' to constructs the distribution of gene expression levels from a given real RNA 
#' sequencing data (single-cell or bulk), and subsequently, simulates a new 
#' dataset from the estimated marginal distributions using Gaussian-copulas to 
#' retain the dependence between genes. It allows simulation of multiple 
#' groups and batches with any required sample size and library size.
#'
#' @docType package
#' @author  Alemu Takele Assefa \email{alemutak@hotmail.com}
#' @author Stijn Hawinkel \email{stijnhawinkel@hotmail.com}
#' 
#' @references
#' \itemize{
#' \item Alemu Takele Assefa, Jo Vandesompele, Olivier Thas. (2020). SPsimSeq: semi-parametric simulation of bulk and single cell RNA sequencing data, \emph{Bioinformatics}, , btaa105, https://doi.org/10.1093/bioinformatics/btaa105
#' }
#' @name SPsimSeq-package  
#' 
NULL

#'Select candidate genes
#'
#'This function can be used to independently select candidate genes from a given real RNA-srq data (bulk/single)
#'for the SPsimSeq simulation. It chooses genes with various chracteristics, such as log-fold-change 
#' above a certain thereshold. 
#' 
#' @param cpm.data logCPM transformed matrix (if log.CPM.transform=FALSE, then it is the source gene expression data)
#' @param lfc.thrld a positive numeric value for the minimum absolute log-fold-change for selecting candidate DE genes in the source data (when group is not NULL and pDE>0)
#' @param t.thrld a positive numeric value for the minimum absolute t-test statistic for the log-fold-changes of genes for selecting candidate DE genes in the source data (when group is not NULL and  pDE>0)
#' @param llStat.thrld a positive numeric value for the minimum squared test statistics from the log-linear model to select candidate DE genes in the source data (when group is not NULL and  pDE>0)
#' containing X as a covariate to select DE genes 
#' @param max.frac.zeror.diff a numeric value >=0 indicating the maximum absolute
#' difference in the fraction of zero counts between the groups for DE genes.
#' @param w a numeric value between 0 and 1. The number of classes to construct the probability distribution will be round(w*n), where n is the total number of samples/cells in a particular batch of the source data
#' @param group a grouping factor
#' @param pDE fraction of DE genes
#' @param n.genes total number of genes
#' @param prior.count a positive constant to be added to the CPM before log transformation, to avoid log(0). The default is 1.
#'
#' @return a list object contating a set of candidate null and non-null genes and additional results
#' @importFrom stats lm glm sd coef
#' @importFrom utils combn
chooseCandGenes <- function(cpm.data, group, lfc.thrld, llStat.thrld, t.thrld, w =w,
                             max.frac.zeror.diff = Inf, pDE, n.genes, prior.count){
  # calculate fold-changes and t-statistics
  logConst =  log(prior.count)
  m.diff  <- apply(cpm.data, 1, function(y){ 
    l.mod  <- lm(y~group)
    t.stat <- max(abs(summary(l.mod)[["coefficients"]][-1, "t value"]))
    #Removed as.numeric
    fc     <- max(abs(coef(l.mod)[-1]))
    frac.z.diff <- max(abs(combn(tapply(y, group, function(yy) mean(yy==logConst)), 2, FUN=diff)))
    c(t.stat = t.stat, fc = fc, frac.z.diff = frac.z.diff)
  })
  null.genes0    <- colnames(m.diff)[m.diff["t.stat",] < t.thrld | m.diff["fc",] < lfc.thrld ]
  nonnull.genes0 <- colnames(m.diff)[(m.diff["t.stat",] >= t.thrld) & 
                                       (m.diff["fc",] >= lfc.thrld) & 
                                       (m.diff["frac.z.diff",] <= max.frac.zeror.diff)]
  #For the ones exceeding the threshold,
  statLLmodel <- if(length(nonnull.genes0)>=1){
     vapply(nonnull.genes0, FUN.VALUE = numeric(1), function(j){
      fits = tapply(cpm.data[j,], group, function(Y){
        mu.hat = mean(Y)
        sig.hat = sd(Y)
        #Bin counts
        countY <- obtCount(Y = Y, w = w)
        #Fit exponential density
        llModel = fitLLmodel(yy = countY, mu.hat = mu.hat, sig.hat = sig.hat, 
                             n = length(Y))
        #define offset of carrier density
        ofs = log(llModel$g0*length(Y))
        list(ofs = ofs, llModel = llModel)
      })
      #Extract counts, offsets and midpoints, and concatenate
      df = data.frame(
        counts = unlist(lapply(fits, function(x) x$llModel$counts)),
        mids = unlist(lapply(fits, function(x) x$llModel$mids)),
        ofs = unlist(lapply(fits, function(x) x$ofs)),
        groups = factor(unlist(lapply(seq_along(fits), function(l) {
          rep(l, length(fits[[l]]$ofs))})))
      )
      l.mod.x = NULL; i = 5
      #The different models to try
      terms = c("mids", "groups", "groups:mids", "I(mids^2)", "groups:I(mids^2)")
      while (is.null(l.mod.x) && i>=3){
        form = paste("counts ~", paste(terms[seq_len(i)], collapse ="+"), "+offset(ofs)")
        l.mod.x <- tryCatch(glm(form, family = "poisson", data = df),
                            error=function(e){}, warning=function(w){})
        i = i-1
      }
      #Return squared sum of test statistics
      sum.square.Z.X = if(!is.null(l.mod.x) && l.mod.x$rank == ncol(l.mod.x$R)){
          Z.X <- summary(l.mod.x)$coefficients[, 3]
          Z.X <- Z.X[grepl(names(Z.X), pattern = "groups")]
          sum(Z.X^2, na.rm = TRUE)
      } else{0} 
      return(sum.square.Z.X)
    })
  } else{
      0
  }
  nonnull.genes <- nonnull.genes0[statLLmodel>=llStat.thrld]
  null.genes    <- c(null.genes0, setdiff(nonnull.genes0, nonnull.genes))
  #Throw warnings where needed
  if((1-pDE)*n.genes > length(null.genes) && length(null.genes)>1 ){
    message("Note: The number of null genes (not DE) in the source data is ",
            length(null.genes),
            " and the number of null genes required to be included in the simulated data is ", 
            round((1-pDE)*(n.genes)),
            ". Therefore, candidiate null genes are sampled with replacement.")
  }
  if(pDE*n.genes > length(nonnull.genes) && length(nonnull.genes)>1){
    message("Note: The number of DE genes detected in the source data is ",
            length(nonnull.genes),
            " and the number of DE genes required to be included in the simulated data is ",
            round(pDE*n.genes),
            ". Therefore, candidiate DE genes are sampled with replacement.")
  }
  if(pDE>0 & length(nonnull.genes)==0){
    warning("No gene met the criterion to be a candidiate DE gene. Perhaps consider
            lowering the 'lfc.thrld' or the 'llStat.thrld' or the 't.thrld'. Consequently,
            all the simulated genes are not DE.")
  }
  list(null.genes = unique(null.genes), nonnull.genes = nonnull.genes)
}

#' A function with S4 dispatching to extract the count matrix
#' @param Y a matrix, data frame, phyloseq object or SingleCellExperiment
#' @param ... additional arguments, currently ignored
#' @return A data matrix with samples in the columns and genes in the rows
#' @rdname extractMat
setGeneric("extractMat", function(Y, ...) standardGeneric("extractMat"))
#' @rdname extractMat
#' @import methods
#' @importFrom SingleCellExperiment counts
setMethod("extractMat", "SingleCellExperiment", function(Y, ...){
  counts(Y)
})
#' @rdname extractMat
setMethod("extractMat", "matrix", function(Y, ...){
  Y
})
#' @rdname extractMat
setMethod("extractMat", "data.frame", function(Y, ...){
  as.matrix(t(Y))
})
#' @rdname extractMat
#' @importFrom phyloseq t taxa_are_rows otu_table
setMethod("extractMat", "phyloseq", function(Y, ...){
  if(!taxa_are_rows(Y)){
    Y = t(Y)
  }
  as(otu_table(Y), "matrix")
})

#' Fast fit Poisson regression
#'
#' @param Ny vector of counts
#' @param x regressor
#' @param degree degree of the polynomial
#' @param offset offset
#' @importFrom stats glm.fit poisson
#' @return see glm.fit
fitPoisGlm = function(Ny, x, degree, offset){
  desMat = buildXmat(x, degree+1)
  glm.fit(y = Ny, x = desMat, family = poisson(), offset = offset)
}

context("Fast fitting a Poisson glm")
y = rpois(20, 2)
x = rnorm(20)
offs = log(runif(20,1,2))
test_that("fitGlmPois does the same as glm",{
    expect_equivalent(fitPoisGlm(y, x, 2, offs)$coef,
                    glm(y~x+I(x^2), family = "poisson", offset = offs)$coef,
                    tolerance = 1e-5)
  expect_equivalent(fitPoisGlm(y, x, 3, offs)$coef,
                    glm(y~x+I(x^2)+I(x^3), family = "poisson", offset = offs)$coef,
                    tolerance = 1e-5)
          })

#' Construct the cumulative density
#'
#' @param densList.ii the estimated density parameters
#' @param exprmt.design experiment configuration
#' @param DE.ind.ii a boolean, is the gene to be DE?
#' @param returnDens A boolean, should densities rather than cumulative 
#' densities be returned?
#'
#' @return The cumulative density
constructDens = function(densList.ii, exprmt.design, DE.ind.ii, 
                         returnDens = FALSE){
    lapply(seq_along(exprmt.design$sub.batchs), function(i){
      batch = exprmt.design$sub.batchs[[i]]
      dl = if(DE.ind.ii){
        densList.ii[[batch]][[exprmt.design$sub.group[[i]]]]
      } else {
        densList.ii[[batch]]
      }
      Coef = dl$coef
      if(is.null(Coef)){
        gy = dl$g0*dl$n
      } else {
        s.mat <- buildXmat(dl$mids, nc = length(Coef))
        gy = dl$g0*dl$n*exp(s.mat %*% Coef)
        id = is.infinite(gy)
        if(any(id)){
          gy[id] = max(gy[!id])
        }
      }
      if(returnDens){
        return(list(gy = c(gy), breaks = dl$breaks, mids = dl$mids))
      } else{
        Gy = cumsum(gy)/sum(gy) 
        return(list(Gy = Gy, breaks = dl$breaks))
      }
    })
  }

#' Sample genes from candidate genes
#'
#' @param pDE fraction of genes to be made DE
#' @param exprmt.design the experiment design
#' @param n.genes the total number of genes required
#' @param null.genes0,nonnull.genes0 Candidate null and non-null genes
#'
#' @return a vector of selected genes
selectGenes <- function(pDE, exprmt.design, n.genes, null.genes0, nonnull.genes0){
  if(pDE>0 && length(unique(exprmt.design$sub.groups))>1){
    null.genes = sample(null.genes0, (1-pDE)*(n.genes), 
                        replace = (1-pDE)*n.genes > length(null.genes0))
    nonnull.genes = if(length(nonnull.genes0)>0){
          sample(nonnull.genes0, pDE*n.genes, 
                 replace = pDE*n.genes > length(nonnull.genes0))
    } else {NULL}
    sel.genes <- c(nonnull.genes, null.genes)
  } else{
    sel.genes <- sample(null.genes0, n.genes, 
                        replace = n.genes > length(null.genes0))
  } 
  names(sel.genes) = sel.genes
  sel.genes
}

context("Gene selection")
genes = paste0("Gene_", seq_len(500))
id = sample(c(TRUE, FALSE), replace = TRUE, length(genes))
genesnull = genes[id]
genesnonnull = genes[!id]
genes2 = paste0("Gene_", seq_len(10))
id2 = sample(c(TRUE, FALSE), replace = TRUE, length(genes2))
genes2null = genes2[id2]
genes2nonnull = genes2[!id2]
batch = sample(LETTERS[1:3], 20, replace = TRUE)
group = sample(1:3, 20, replace = TRUE)
Design = configExperiment(batch.config = c(0.5, 0.3, 0.2), 
                          group.config = c(0.6, 0.4), tot.samples = 400, 
                          batch = batch, group = group)
test_that("Gene selection selects correct number of genes",{
    expect_identical(
      length(selectGenes(0.2, Design, 200, genesnull, genesnonnull)), 200L
      )
    expect_identical(
      length(selectGenes(0, Design, 200, genesnull, genesnonnull)), 200L
    )
    expect_identical(
      length(selectGenes(1, Design, 200, genesnull, genesnonnull)), 200L
    )
    expect_identical(
      length(selectGenes(0.5, Design, 200, genes2null, genes2nonnull)), 200L
    )
    expect_identical(
      length(selectGenes(0, Design, 200, genes2null, genes2nonnull)), 200L
    )
    expect_identical(
      length(selectGenes(1, Design, 200, genes2null, genes2nonnull)), 200L
    )
          })

#' Estimate log-normal distribution for the library sizes 
#'
#' @param LS observed library sizes
#' @param batch batches
#' 
#' @return Estimated log-normal parameter library sizes
#' @importFrom fitdistrplus fitdist
estLibSizeDistr = function(LS, batch){
  tapply(LS, batch, function(L.b){
      fitdist(L.b, distr = "lnorm")$estimate 
  })
}

#' Extract data and iterate over batches to estimate zero probability models
#'
#' @param s.data,cpm.data raw and transformed data 
#' @param batch the batch vector
#' @param n.mean.class see zeroProbModel
#' @param minFracZeroes minimum fraction of zeroes before zero-inflation is applied
#'
#' @return a list of binomial regression parameters
fracZeroLogitModel <- function(s.data, batch, cpm.data, n.mean.class, 
                               minFracZeroes){
  LS = colSums(s.data)
  zeroMat = s.data == 0
  zeroModels = tapply(colnames(s.data), batch, function(coln){
    zeroModel = zeroProbModel(cpm.data = cpm.data[, coln], logL = log(LS[coln]), 
                  zeroMat[, coln],  n.mean.class = n.mean.class)
    #Calculate zero fractions of each gene within the batches
    zeroFrac = rowMeans(zeroMat[, coln])
    #Calculate gene-wise means
    geneMeans = rowMeans(cpm.data[, coln])
    #Retain the means of the genes exceeding the zero fraction threshold
    meansLarge = geneMeans[zeroFrac > minFracZeroes]
    list(zeroModel = zeroModel, meansLarge = meansLarge)
  })
}

#' Calculates counts per millions of reads, possibly with log-transform
#'
#' @param X raw data matrix
#' @param const.mult a constant to multiply with
#' @param prior.count prior count to be added to the zeroes
#'
#' @return a normalized data matrix
#' @importFrom edgeR calcNormFactors
calculateCPM <- function(X, const.mult, prior.count){
    norm.factors = edgeR::calcNormFactors(X)*colSums(X)
    cpm = X %*% diag(1/(norm.factors)) * const.mult
    colnames(cpm) = colnames(X)
    log(cpm + prior.count)
}

#' Generate library sizes from log-normal
#'
#' @param fit.ln the library size model
#' @param exprmt.design the design
#'
#' @return The generated libray sizes per batch and group
#' @importFrom stats rlnorm
genLibSizes = function(fit.ln, exprmt.design){
  n.batch = rowSums(exprmt.design$exprmt.config)
  lapply(unique(exprmt.design$sub.batchs), function(b){
    rlnorm(n.batch[b], fit.ln[[b]][["meanlog"]], fit.ln[[b]][["sdlog"]])
  })
}

#' Neuroblastoma NGP cells single-cell RNA-seq.
#'
#'  It was retrieved from [1] (GEO accession GSE119984): This dataset is 
#'  generated for a cellular perturbation experiment on the C1 instrument 
#'  (SMARTer protocol) [1]. This total RNA-seq dataset contains 
#'  83 NGP neuroblastoma cells, of which 31 were treated with 
#'  nutlin-3 and the other 52 cells were treated with vehicle (controls). 
#' 
#' @docType data
#' 
#' @format A SingleCellExperiment object
#' 
#' @keywords dataset
#' 
#' @references 1 - Verboom, K., Everaert, C., Bolduc, N., Livak, K. J., Yigit, N., Rombaut, D., ... & Speleman, F. (2019). SMARTer single cell total RNA sequencing. Nucleic Acids Research, 47(16), e93-e93.
#' 
#' \describe{
#'   \item{SingleCellExperiment}{counts + gene info + cell infro} 
#' }
#'  
#' @source GEO accession GSE119984
#' @examples 
#' data("scNGP.data")
#' scNGP.data
"scNGP.data"

#'An auxialiary function to quickly construct the polyomial matrix,
#'using Horner's rule
#'
#'@param x The base
#'@param nc the number of columns
#'
#'@return A matrix with increasing powers of x in the columns
buildXmat = function(x, nc){
  x.mat <- matrix(1, nrow = length(x), ncol = nc)
  for(k in seq_len(nc)[-1]){
    x.mat[,seq(k,nc)] <- x.mat[,seq(k,nc)]*x
  }
  x.mat
}

context("Building the design matrix")
x = rnorm(5)
test_that("buildXmat does the same as model.matrix",{
    expect_equivalent(model.matrix(~I(x)+I(x^2)),
                    buildXmat(x, 3))
    expect_equivalent(model.matrix(~I(x)+I(x^2)+I(x^3)),
                    buildXmat(x, 4))
    expect_equivalent(model.matrix(~I(x)+I(x^2)+I(x^3)+I(x^4)),
                    buildXmat(x, 5))
          })

#' Fit log linear model for each gene
#' 
#' @param yy a list object contating a result from obtCount() function for a single gene 
#' @param mu.hat,sig.hat Carrier density estimators
#' @param n number of observations
#' 
#' @return a list object containing the fitted log linear model and carrier density
#' @importFrom stats pnorm
fitLLmodel <- function(yy, mu.hat, sig.hat, n){
  #Evaluate carrier density
  g0 = diff(pnorm(yy$breaks, mean = mu.hat, sd = sig.hat))
  g0[g0==0] = .Machine$double.eps
  ofs = log(g0*n)
  llm = NULL; degree = 4
  while(is.null(llm) && (degree >= 1)){
  llm <- tryCatch(fitPoisGlm(yy$counts, yy$mids, degree, offset = ofs), 
                  error = function(e){}, warning = function(w){}) 
  if(!is.null(llm) && llm$rank != ncol(llm$R)){
    llm = NULL
  }
  degree = degree - 1
  }
  return(c(yy, list(g0 = g0, coef = llm$coef, n = n)))
}

#' A function to obtain copulas or uniform random variables
#'
#' @param cpm.data the transformed data matrix
#' @param batch the batch indicators
#'
#' @return The estimated correlation matrices per batch
#' @importFrom WGCNA cor
#' @importFrom limma voom
obtCorMatsBatch <- function(cpm.data, batch){
  tapply(colnames(cpm.data), batch,  function(coln){ 
    data = cpm.data[, coln]
    voom.res <- limma::voom(data)
    W <- voom.res$weights
    WGCNA::cor(x=t(data), weights.x = t(W), use = "pairwise.complete.obs")
  }) 
}

#' Gene level param estimates for density estimation
#'
#' @param cpm.data.i full vector of genewise observation
#' @param de.ind a boolean, is the gene to be DE?
#' @param model.zero.prob a boolean, should zero-density be modelled?
#' @param w weight
#' @param batch,group batch and grouping vectors 
#' @param prior.count the prior count for the cpm transofrm
#'
#' @return list of density estimates
geneParmEst <- function(cpm.data.i, batch, group, prior.count = prior.count,
                          de.ind, model.zero.prob, w){
  min.val = log(prior.count)
  tapply(seq_along(cpm.data.i), batch, function(i){ 
    #When not DE, overal density estimation
    if(!de.ind){
      parmEstDensVec(cpm.data.i[i], model.zero.prob, min.val, w = w)
    } else {
      #Otherwise separately per group
     tapply(cpm.data.i[i], group[i], function(Y){
       parmEstDensVec(Y, model.zero.prob, min.val, w = w)
     }) 
    }
  })
}
#' Density estimation on a single vector
#'
#' @param Y0 the vector of observations
#' @param model.zero.prob,min.val,w see geneParmEst()
#' @param prev.min.val minimum prevalence of minimum values
#' @param min.count.nonnull minimum count for estimation
#'
#' @return density estimates
parmEstDensVec = function(Y0, model.zero.prob, min.val, w, prev.min.val = 0.25, 
                           min.count.nonnull = 3){
  #Subset vector
  Y = if(model.zero.prob & mean(Y0==min.val) > prev.min.val){
    Y0[Y0>min.val]
  } else Y0
  n = length(Y)
  #Fit normal carrier density
  mu.hat = mean(Y)
  sig.hat = sd(Y)
  #Bin counts
  countY <- obtCount(Y = Y, w = w)
if(sum(Y > min.val) > min.count.nonnull){
  #Fit exponential density
  fitLLmodel(yy = countY, mu.hat = mu.hat, sig.hat = sig.hat, n = n)
} else{
  g0 = diff(pnorm(countY$breaks, mean = mu.hat, sd = sig.hat))
  g0[g0==0] = .Machine$double.eps
  c(countY, list(g0 = g0, n = n))
  }
}

#' Configure experiment
#'
#' @param batch.config a numerical vector for the marginal fraction of samples in each batch.
#' The number of batches to be simulated is equal to the size of the vector.
#' All values must sum to 1.
#' @param group.config a numerical vector for the marginal fraction of samples in each group.
#' The number of groups to be simulated is equal to the size of the vector. All values must sum to 1.
#' @param tot.samples total number of samples to be simulated.
#' @param batch,group batch and grouping vectors
#'
#' @return a list object contating the number of groups and batches to be simukated,
#' and the experiment configurartion
#' @export
#' @examples
#' batch = sample(LETTERS[seq_len(3)], 20, replace = TRUE)
#' group = sample(seq_len(3), 20, replace = TRUE)
#' #---- a design with a total of 10 samples/cells from 1 batch and 1 group
#' configExperiment(batch.config=1, group.config=1, tot.samples=10,
#' batch = batch, group = group)
#'
#' #---- a design with a total of 20 samples/cells from 1 group and 2 batchs with
#' # batch 1 has 15 samples/cells and batch 2 has 5
#' configExperiment(batch.config = c(15/20, 5/20), group.config = 1,
#' tot.samples = 20, batch = batch, group = group)
#'
#' #---- a design with a total of 20 samples/cells from 1 batch and 2 groups with
#' # group 1 has 10 samples/cells and batch 2 has 10
#' configExperiment(batch.config=1, group.config=c(0.5, 0.5), tot.samples=20,
#' batch = batch, group = group)
#'
#' #---- a design with a total of 30 samples/cells from 2 groups with group 1 has 15 samples
#' # and group 2 has 15, and  three batchs with batch 1,2, and 3 have 5, 10, and 15 samples/cells,
#' # respectively.
#' configExperiment(batch.config = c(5/30, 10/30, 15/30), group.config = c(0.5, 0.5),
#'  tot.samples = 30, batch = batch, group = group)
configExperiment <- function(batch.config, group.config, tot.samples, batch, group){
  #Sort, such that largest groups and largest batches match with simulation
  batch.config = sort(batch.config, decreasing = TRUE)
  group.config = sort(group.config, decreasing = TRUE)
  #Assign names, and select most abundant groups and batches to sample from
  namesBatch = names(sort(table(batch), decreasing = TRUE))[seq_along(batch.config)]
  namesGroup = names(sort(table(group), decreasing = TRUE))[seq_along(group.config)]
  names(batch.config) <- namesBatch
  names(group.config) <- namesGroup
  #Configure the experiment
  exprmt.config <- round(tcrossprod(batch.config, group.config)*tot.samples)
  rownames(exprmt.config) <- names(batch.config)
  colnames(exprmt.config) <- names(group.config)
  sub.batchs = rep(rep(namesBatch, ncol(exprmt.config)), times = c(exprmt.config))
  sub.groups = rep(namesGroup, times = colSums(exprmt.config))
  list(exprmt.config = exprmt.config, sub.batchs = sub.batchs,
       sub.groups = sub.groups)
}

context("Configuring the experiment")
batch = sample(LETTERS[1:3], 20, replace = TRUE)
group = sample(1:3, 20, replace = TRUE)
test_that("configExperiment yields desired output",{
    expect_equivalent(configExperiment(batch.config=1, group.config=1, tot.samples=100, 
                                            batch = batch, group = group)$exprmt.config,
    100)
    expect_equivalent(configExperiment(batch.config=c(0.5,0.3,0.2), group.config=c(0.6,0.4), tot.samples = 400, 
                                             batch = batch, group = group)$exprmt.config,
                            matrix(c(120, 72, 48, 80, 48, 32), nrow = 3, ncol = 2))
          })

#' A function to simulate bulk or single cell RNA sequencing data
#'
#' @description This function simulates (bulk/single cell) RNA-seq dataset from semi-parametrically estimated distributions of gene expression levels in a given real source RNA-seq dataset
#'
#' @param n.sim an integer for the number of simulations to be generated
#' @param s.data a real source dataset (a SingleCellExperiment class or a matrix/data.frame of counts with genes in rows and samples in columns)
#' @param batch NULL or a vector containing batch indicators for each sample/cell in the source data
#' @param group NULL or a vector containing group indicators for each sample/cell in the source data
#' @param n.genes a numeric value for the total number of genes to be simulated
#' @param cand.DE.genes a list object contatining canidiate null and non-null (DE/predictor) genes.
#' If NULL (the default), an internal function determines candidate genes based on log-fold-change and other statistics.
#' The user can also pass a list of canidate null and non-null genes (they must be disjoint). In particular, the list should contain
#' two character vectors (for the name of the features/genes in the source data) with names 'null.genes' and 'nonnull.genes'.
#' For example, cand.DE.genes=list(null.genes=c('A', 'B'), nonnull.genes=c('C', 'D')).
#' @param pDE a numeric value between 0 and 1 indicating the desired fraction of DE genes in the simulated data
#' @param batch.config a numerical vector containing fractions for the composition of samples/cells per batch. The fractions must sum to 1. The number of batches to be simulated is equal to the size of the vector. (Example, batch.config=c(0.6, 0.4)   means simulate 2 batches with 60\% of the simulated samples/cells in batch 1 and the rest 40\% in the second batch. Another example,  batch.config=c(0.3, 0.35, 0.25) means simulate 3 batches with the first, second and third batches contain 30\%, 35\% and 25\% of the samples/cells, respectively).
#' @param group.config a numerical vector containing fractions for the composition of samples/cells per group. Similar definition to 'batch.config'. The number of groups to be simulated is equal to the size of the vector. The fractions must sum to 1.
#' @param lfc.thrld a positive numeric value for the minimum absolute log-fold-change for selecting
#' candidate DE genes in the source data (when group is not NULL, pDE>0 and cand.DE.genes is NULL)
#' @param t.thrld a positive numeric value for the minimum absolute t-test statistic for the
#' log-fold-changes of genes for selecting candidate DE genes in the source data (when group
#' is not NULL, pDE>0 and cand.DE.genes is NULL)
#' @param llStat.thrld a positive numeric value for the minimum squared test statistics from
#' the log-linear model to select candidate DE genes in the source data (when group is not NULL,
#' pDE>0 and cand.DE.genes is NULL)
#' @param model.zero.prob a logical value whether to model the zero expression probability separately (suitable for simulating of single-cell RNA-seq data or zero-inflated data)
#' @param tot.samples a numerical value for the total number of samples/cells to be simulated.
#' @param result.format a character value for the type of format for the output. Choice can be 'SCE' for SingleCellExperiment class or "list" for a list object that contains the simulated count, column information and row information.
#' @param return.details a logical value. If TRUE, detailed results including estimated parameters and densities will be returned
#' @param genewiseCor a logical value, if TRUE (default) the simulation will retain the gene-to-gene correlation structure of the source data using Gausian-copulas . Note that if it is TRUE, the program will be slow or it may fail for a limited memory size.
#' @param prior.count a positive constant to be added to the CPM before log transformation, to avoid log(0). The default is 1.
#' @param log.CPM.transform a logical value. If TRUE, the source data will be transformed into log-(CPM+const) before estimating the probability distributions
#' @param lib.size.params NULL or a named numerical vector containing parameters for simulating library sizes from log-normal distribution. If lib.size.params =NULL (default),
#' then the package will fit a log-normal distribution for the library sizes in the source data to simulate new library sizes.
#' If the user would like to specify the parameters of the log-normal distribution for the desired library sizes,
#' then the log-mean and log-SD params of rlnorm() functions can be passed using this argument.
#' Example, lib.size.params = c(meanlog=10, sdlog=0.2). See also ?rlnorm.
#' @param variable.lib.size a logical value. If FALSE (default), then the expected library sizes are simulated once and remains the same for every replication (if n.sim>1).
#' @param verbose a logical value, if TRUE a message about the status of the simulation will be printed on the console
#' @param w see ?hist
#' @param const.mult A constant by which the count are scaled. Usually 1e6 to get CPM
#' @param n.mean.class a fraction of the number of genes
#' for the number of groups to be created for the mean log CPM of genes
#' @param minFracZeroes minimum fraction of zeroes before a zero inflation model
#' is fitted
#'
#' @return a list of SingleCellExperiment/list objects each containing simulated counts (not normalized), smple/cell level information in colData, and gene/feature level information in rowData.
#'
#' @details This function uses a specially designed exponential family for density estimation
#' to constructs the distribution of gene expression levels from a given real gene expression data
#' (e.g. single-cell or bulk sequencing data), and subsequently, simulates a new
#' from the estimated distributions.#'
#' For simulation of single-cell RNA-seq data (or any zero inflated gene expression data), the programm
#' involves an additional step to explicitly account for the high abundance of zero counts (if required).
#' This step models the probability of zero counts as a function the mean
#' expression of the gene and the library size of the cell (both in log scale) to add excess zeros.
#' This can be done by using \emph{model.zero.prob=TRUE}. Note that,
#' for extremly large size data, it is recomended to use a random sample of cells to
#' reduce computation time. To enable this, add the argument \emph{subset.data=TRUE} and you
#' can specify the number of cells to be used using \emph{n.samples} argument.
#' For example \emph{n.samples=400}.
#' Given known groups of samples/cells in the source data, DGE is simulated by independently
#' sampling data from distributions constructed for each group seprately. In particular, this procedure is
#' applied on a set of genes with absolute log-fold-change in the source data more than a given threshold (\emph{lfc.thrld}).
#' Moreover, when  the source dataset involves samples/cells processed in different batches, our
#' simulation procedure incorporates this batch effect in the simulated data, if required.
#' Different experimental designs can be simulated using the group and batch configuration arguments to
#' simulate biologica/experimental conditions and batchs, respectively.
#' Also, it is important to filter the source data so that genes with suffient expression will be used to
#' estimate the probability distributions.
#'
#' @references
#' \itemize{
#' \item Assefa, A. T., Vandesompele, J., & Thas, O. (2020). SPsimSeq: semi-parametric simulation of bulk and single cell RNA sequencing data. \emph{Bioinformatics}, doi: https://doi.org/10.1093/bioinformatics/btaa105.
#' \item Efron, B., & Tibshirani, R. (1996). Using specially designed exponential families for density estimation. \emph{The Annals of Statistics}, 24(6), 2431-2461.
#' }
#'
#' @examples
#' #----------------------------------------------------------------
#' # Example 1: simulating bulk RNA-seq
#'
#' # load the Zhang bulk RNA-seq data (available with the package)
#' data("zhang.data.sub")
#'
#' zhang.counts <- zhang.data.sub$counts
#' MYCN.status  <- zhang.data.sub$MYCN.status
#'
#' # We simulate only a single data (n.sim = 1) with the following property
#' # - 100 genes ( n.genes = 100)
#' # - 8 samples (tot.samples = 8)
#' # - the samples are equally divided into 2 groups each with 4 samples
#' #   (group.config = c(0.5, 0.5))
#' # - all samples are from a single batch (batch = NULL, batch.config = 1)
#' # - we add 10% DE genes (pDE = 0.1)
#' # - the DE genes have a log-fold-change of at least 0.5 in
#' #   the source data (lfc.thrld = 0.5)
#' # - we do not model the zeroes separately, they are the part of density
#' #    estimation (model.zero.prob = FALSE)
#' # We keep the numbers of genes and samples low to limit compile time
#'
#' # simulate data
#' sim.data.bulk <- SPsimSeq(n.sim = 1, s.data = zhang.counts,
#'                           group = MYCN.status, n.genes = 50, batch.config = 1,
#'                           group.config = c(0.5, 0.5), tot.samples = 8,
#'                           pDE = 0.1, lfc.thrld = 0.5, result.format = "list")
#' head(sim.data.bulk$counts[[1]][, seq_len(5)])  # count data
#' head(sim.data.bulk$colData)        # sample info
#' head(sim.data.bulk$rowData)        # gene info
#' #----------------------------------------------------------------
#' # Example 2: simulating single cell RNA-seq from a single batch (read-counts)
#' # we simulate only a single scRNA-seq data (n.sim = 1) with the following property
#' # - 100 genes (n.genes = 100)
#' # - 6 cells (tot.samples = 6)
#' # - the cells are equally divided into 2 groups each with 3 cells
#' #   (group.config = c(0.5, 0.5))
#' # - all cells are from a single batch (batch = NULL, batch.config = 1)
#' # - we add 10% DE genes (pDE = 0.1)
#' # - the DE genes have a log-fold-change of at least 0.5
#' # - we model the zeroes separately (model.zero.prob = TRUE)
#' # - the ouput will be in SingleCellExperiment class object (result.format = "SCE")
#'
#' library(SingleCellExperiment)
#'
#' # load the NGP nutlin data (availabl with the package, processed with
#' # SMARTer/C1 protocol, and contains read-counts)
#' data("scNGP.data")
#' treatment <- ifelse(scNGP.data$characteristics..treatment=="nutlin",2,1)
#' # simulate data (we simulate here only a single data, n.sim = 1)
#' sim.data.sc <- SPsimSeq(n.sim = 1, s.data = scNGP.data, group = treatment,
#'  n.genes = 40, batch.config = 1, group.config = c(0.5, 0.5),
#'  tot.samples = 6, pDE = 0.1, lfc.thrld = 0.5, model.zero.prob = TRUE,
#'                     result.format = "SCE")
#' @export
SPsimSeq <- function(n.sim = 1, s.data, batch = rep(1, ncol(s.data)),
                     group = rep(1, ncol(s.data)),
                     n.genes = 1000, batch.config = 1, group.config = 1,
                     pDE = 0.1, cand.DE.genes = NULL, lfc.thrld = 0.5,
                     t.thrld = 2.5, llStat.thrld = 5, tot.samples = ncol(s.data),
                     model.zero.prob = FALSE, genewiseCor = TRUE,
                     log.CPM.transform = TRUE, lib.size.params = NULL,
                     variable.lib.size = FALSE, w = NULL,
                     result.format = "SCE", return.details = FALSE,
                     verbose = TRUE, prior.count = 1, const.mult = 1e6,
                     n.mean.class = 0.2, minFracZeroes = 0.25)
{
  #DATA EXTRACTION
  s.data = extractMat(s.data)
  #INPUT CHECKS
  checkInputs = checkInputValidity(s.data = s.data, group = group, batch = batch,
                                    group.config = group.config, batch.config = batch.config,
                                    w = w, log.CPM.transform = log.CPM.transform, pDE = pDE,
                                    prior.count = prior.count, lib.size.params = lib.size.params,
                                    llStat.thrld = llStat.thrld, result.format = result.format)
  #CPM TRANSFORM
  cpm.data <- if(log.CPM.transform){
    # calculate log CPM
     calculateCPM(s.data, prior.count = prior.count,
                             const.mult = const.mult)
  } else s.data
  #PARAMETER ESTIMATION
  # Estimate library size distributions
  LS = colSums(s.data) #Library sizes
  if(variable.lib.size && is.null(lib.size.params)){
    if(verbose){message("Fitting library size distirbution ...")}
    lib.size.params <- estLibSizeDistr(LS = LS, batch = batch)
  }
  # Estimate correlation matrices
  if(genewiseCor){
    if(verbose) {message("Estimating featurewise correlations ...")}
    corMats.batch <- obtCorMatsBatch(cpm.data = cpm.data, batch = batch)
  }
  #Find candidate DE genes
  if(is.null(cand.DE.genes)){
    if(verbose) {message("Selecting candidate DE genes ...")}
    cand.DE.genes = if(length(unique(group))>1){
      chooseCandGenes(cpm.data = cpm.data, group = group, prior.count = prior.count,
                      lfc.thrld = lfc.thrld, t.thrld = t.thrld, w = w,
                      llStat.thrld = llStat.thrld, pDE = pDE, n.genes = n.genes)
    } else {
      list(null.genes = rownames(s.data))
    }
  }
  #Selected genes
  null.genes0    <- cand.DE.genes$null.genes
  nonnull.genes0 <- cand.DE.genes$nonnull.genes
  allGenes = c(null.genes0, nonnull.genes0)
  names(allGenes) = allGenes
  # Fit logistic regression for the probability of zeros
  if(all(s.data!=0)) model.zero.prob = FALSE
  if(model.zero.prob){
    if(verbose) {message("Fitting zero probability model ...")}
    fracZero.logit.list <- fracZeroLogitModel(s.data = s.data, batch = batch,
                                              cpm.data = cpm.data,
                                              n.mean.class = n.mean.class,
                                              minFracZeroes = minFracZeroes)
  }
  # Estimate batch specific densities
  if(verbose) {message("Estimating densities ...")}
  densList <- lapply(allGenes, function(gene){
    geneParmEst(cpm.data.i = cpm.data[gene, ], batch = batch, group = group,
                de.ind = gene %in% nonnull.genes0, prior.count = prior.count,
                model.zero.prob = model.zero.prob, w = w)
  })
  #SIMULATION
  ## EXPERIMENT CONFIGURATION
  if(verbose) {message("Configuring design ...")}
  exprmt.design <- configExperiment(batch.config = batch.config,
                                    group.config = group.config,
                                    tot.samples = tot.samples, batch = batch,
                                    group = group)
  ## PREPARE THE DENSITIES
  prepDens <- lapply(allGenes, function(gene){
    constructDens(densList.ii = densList[[gene]],
                 DE.ind.ii = gene %in% nonnull.genes0,
                 exprmt.design = exprmt.design)
  })
  ## DATA GENERATION
  if(verbose) {message("Simulating data ...")}
  sim.data.list <- lapply(seq_len(n.sim), function(h){
    if(verbose) {message(" ...", h, " of ", n.sim)}
    #Sample libray sizes
    samLS <- if(variable.lib.size & log.CPM.transform){
       genLibSizes(fit.ln = lib.size.params, exprmt.design = exprmt.design)
    } else sample(LS, tot.samples, replace = tot.samples > length(LS))
    #Sample copula
    copSam = genCopula(corMats.batch, exprmt.design = exprmt.design)
    # sample DE and null genes
    selctGenes <- selectGenes(pDE = pDE, exprmt.design = exprmt.design, n.genes = n.genes,
                                 null.genes0 = null.genes0, nonnull.genes0 = nonnull.genes0)
    #Generate data
    sim.dat <- vapply(selctGenes, function(gene){
      SPsimPerGene(cumDens = prepDens[[gene]],
                   sel.genes.ii = gene, const.mult = const.mult,
                   exprmt.design = exprmt.design,
                   log.CPM.transform = log.CPM.transform,
                   LL = samLS, copSam = copSam,
                   prior.count = prior.count, model.zero.prob = model.zero.prob,
                   fracZero.logit.list = fracZero.logit.list)
    }, FUN.VALUE = numeric(tot.samples))
    sim.data.h <- prepareSPsimOutputs(sim.dat = t(sim.dat), exprmt.design = exprmt.design,
                        DE.ind = selctGenes %in% nonnull.genes0,
                        result.format = result.format, LL = samLS)
    return(sim.data.h)
  })
  if(return.details){
    details = list("densList" = densList,
                   "fracZero.logit.list" = if(model.zero.prob) fracZero.logit.list else NULL,
                   "exprmt.design" = exprmt.design,
                   "corMats.batch" = if(genewiseCor) corMats.batch else NULL,
                   "cand.DE.genes" = cand.DE.genes,
                   "lib.size.params" = lib.size.params
                   )
    list("sim.data.list" = sim.data.list, "detailed.results" = details)
  }else{
    sim.data.list
  }

}

context("SPsimSeq input")
data("zhang.data.sub")
zhang.counts <- zhang.data.sub$counts
MYCN.status  <- zhang.data.sub$MYCN.status
test_that("SPsimSeq throws errors for wrong input types", {
  expect_error(SPsimSeq(n.sim = 1, s.data = zhang.counts,
                        group = MYCN.status, n.genes = 2000, batch.config = 1,
                        group.config = c(0.5, 0.5), tot.samples = 20,
                        pDE = 0.1, lfc.thrld = 0.5, result.format = "data.frame"))
  expect_error(SPsimSeq(n.sim = 1, s.data = zhang.counts,
                        group = MYCN.status, n.genes = 2000, batch.config = 1,
                        group.config = c(0.25, 0.5), tot.samples = 20,
                        pDE = 0.1, lfc.thrld = 0.5, result.format = "list"))
  expect_error(SPsimSeq(n.sim = 1, s.data = zhang.counts,
                        group = MYCN.status, n.genes = 2000, batch.config = 0.9,
                        group.config = c(0.5, 0.5), tot.samples = 20,
                        pDE = 0.1, lfc.thrld = 0.5, result.format = "list"))
  expect_error(SPsimSeq(n.sim = 1, s.data = zhang.counts,
                        group = MYCN.status, n.genes = 2000, batch.config = 1,
                        group.config = c(0.5, 0.5), tot.samples = 20,
                        pDE = 20, lfc.thrld = 0.5, result.format = "list"))
  expect_error(SPsimSeq(n.sim = 1, s.data = zhang.counts,
                        group = MYCN.status, n.genes = 2000, batch.config = 1,
                        group.config = c(0.1, 0.1, 0.5, 0.3), tot.samples = 20,
                        pDE = 0.2, lfc.thrld = 0.5, result.format = "list"))
  expect_error(SPsimSeq(n.sim = 1, s.data = zhang.counts,
                        group = MYCN.status, n.genes = 2000, group.config = 1,
                        batch.config = c(0.1, 0.1, 0.5, 0.3), tot.samples = 20,
                        pDE = 0.2, lfc.thrld = 0.5, result.format = "list"))
  expect_error(SPsimSeq(n.sim = 1, s.data = zhang.counts,
                        group = MYCN.status, n.genes = 2000, batch.config = 1,
                        group.config = c(0.5, 0.5), tot.samples = 20,
                        pDE = 0.2, lfc.thrld = 0.5, result.format = "list",
                        lib.size.params = c(10, 5)))
})

test_that("SPsimSeq throws warnings when log-fold change threshold too high", {
  expect_warning(SPsimSeq(n.sim = 1, s.data = zhang.counts,
                        group = MYCN.status, n.genes = 20, batch.config = 1,
                        group.config = c(0.5, 0.5), tot.samples = 8,
                        pDE = 0.1, lfc.thrld = 50, result.format = "list"))
})

context("Output of the SPsimSeq function")
data("zhang.data.sub")
zhang.counts <- zhang.data.sub$counts
MYCN.status  <- zhang.data.sub$MYCN.status
test_that("SPsimSeq returns correct output types", {
  expect_type(SPsimSeq(n.sim = 1, s.data = zhang.counts,
                          group = MYCN.status, n.genes = 10, batch.config = 1,
                          group.config = c(0.5, 0.5), tot.samples = 4,
                          pDE = 0.1, lfc.thrld = 0.5, result.format = "list"),
              "list")
  expect_s4_class(SPsimSeq(n.sim = 1, s.data = zhang.counts,
                       group = MYCN.status, n.genes = 10, batch.config = 1,
                       group.config = c(0.5, 0.5), tot.samples = 4,
                       pDE = 0.1, lfc.thrld = 0.5, result.format = "SCE")[[1]],
              "SingleCellExperiment")
})

#' Return ID for observations to be set to zero
#'
#' @param fracZero.logit.list The estimated zero model
#' @param logLL the logged library sizes
#' @param gene the gene name
#'
#' @return A boolean, should a zero be introduced or not?
#' @importFrom stats rbinom
samZeroID = function(fracZero.logit.list, logLL, gene){
  if(gene %in% names(fracZero.logit.list$meansLarge)){
    meansLarge = rep(fracZero.logit.list$meansLarge[gene], length(logLL))
    desMat = model.matrix(~meansLarge*logLL)
    zeroProb = expit(desMat %*% fracZero.logit.list$zeroModel)
    as.logical(rbinom(1, 1, zeroProb))
  } else FALSE
}
#' Evaluate the expit function
#' @param x the argument
#' @return the expit of the argument
expit = function(x){
  exp(x)/(1+exp(x))
}

#' A function to prepare outputs
#' @param sim.dat The simulated data
#' @param exprmt.design the design 
#' @param DE.ind the differential abundance indicator
#' @param result.format the desired output format
#' @param LL simulated library sizes
#' 
#' @return the data in the desired format
#'
#' @importFrom SingleCellExperiment SingleCellExperiment
prepareSPsimOutputs <- function(sim.dat, exprmt.design, DE.ind, result.format, 
                                LL){
  sim.dat[is.na(sim.dat)] <- 0
  source.ID = rownames(sim.dat)
  rownames(sim.dat) <- paste0("Gene_", seq_len(nrow(sim.dat)))
  colnames(sim.dat) <- paste0("Sample_", seq_len(ncol(sim.dat)))
  col.data <- data.frame("Batch" = exprmt.design$sub.batchs,
             "Group"  = exprmt.design$sub.groups,
             "sim.Lib.Size" = LL,
             row.names = colnames(sim.dat))
  row.data <- data.frame("DE.ind" = DE.ind, row.names = rownames(sim.dat),
                         "source.ID" = source.ID)
  sim.data = if(result.format == "SCE"){
     SingleCellExperiment(assays = list(counts = sim.dat),
                          colData = col.data, rowData = row.data)
    } else if(result.format == "list"){
      list("counts" = sim.dat, "colData" = col.data, "rowData" = row.data)
    }
  return(sim.data)
}

#' Generate a copula instance
#'
#' @param exprmt.design Number of batches, and batch vector
#' @param corMats List of correlation matrices
#'
#' @return a list of copula instances
#' @importFrom mvtnorm rmvnorm
genCopula = function(corMats, exprmt.design){
  n.batch = rowSums(exprmt.design$exprmt.config)
  copList = lapply(names(n.batch), function(b){
    Z       <- rmvnorm(n = n.batch[b], sigma = corMats[[b]])
    Cpl     <- pnorm(Z)
    colnames(Cpl) <- rownames(corMats[[b]])
    t(Cpl)
  })
  Reduce(copList, f = cbind)
}

---
output: 
  BiocStyle::html_document:
    toc: true
    keep_md: true
vignette: >
  %\VignetteIndexEntry{Manual for the SPsimSeq package: semi-parametric simulation for bulk and single cell RNA-seq data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: references.bib
---

# Contents

\setcounter{tocdepth}{5}
\tableofcontents

# Introduction to SPsimSeq

SPsimSeq is a semi-parametric simulation procedure for simulating bulk and single-cell RNA-seq data. It is designed to maximally retain the characteristics of real RNA-seq data with reasonable flexibility to simulate a wide range of scenarios. In a first step, the logarithmic counts per millions of reads (log-CPM) values from a given real dataset are used for semi-parametrically estimating gene-wise distributions and the between-genes correlation structure. In particular, the estimation of the probability distributions uses the fast log-linear model-based density estimation approach developed by @efron1996using and @lindsey1974construction. The method makes use of the Gaussian-copulas [@Cario97modelingand] to retain the between-genes correlation structure, as implemented by @SHawinkel2019 for microbiome data simulation. Arbitrarily large datasets, with realistically varying library sizes, can be sampled from these distributions while maintaining the correlation structure between the genes. Our method has an additional step to explicitly account for the high abundance of zero counts, typical for single-cell RNA-seq data. This step models the probability of zero counts as a function of the mean expression of the gene and the library size (read depth) of the cell (both in log scale). Zero counts are then added to the simulated data such that the observed relationship (zero probability to mean expression and library size) is maintained. In addition, our method simulates DE by separately estimating the distributions of the gene expression from the different populations (for example treatment groups) in the source data, and subsequently sampling a new dataset from each group. The details of the SPsimSeq procedures, implementations and benchmarking results can be found in the supplementary file.  

In this documentation, we will demonstrate SPsimSeq for simulating bulk and single-cell RNA-seq, data subsequently compare the characteristics of the simulated data with the real source data.

# Installing SPsimSeq
The package can be installed and loaded using the following commands:

```{r githubInstall, eval=FALSE}
## Install SPsimSeq
library(devtools)
install_github("CenterForStatistics-UGent/SPsimSeq")
```

or from BioConductor

```{r BiocInstall, eval=FALSE}
## Install SPsimSeq
library(BiocManager)
BiocManager::install("SPsimSeq")
```

```{r loadSPsimSeq}
# load package
library(SPsimSeq) 
```

# Demonstration
## Example 1: simulating bulk RNA-seq
**Zhang RNA-seq data [@Zhang241190]:** The data contains 498 neuroblastoma tumors. In short, unstranded poly(A)+ RNA sequencing was performed on the HiSeq 2000 instrument (Illumina). Paired-end reads with a length of 100 nucleotides were obtained. To quantify the full transcriptome, raw fastq files were processed with Kallisto v0.42.4 (index build with GRCh38-Ensembl v85). The pseudo-alignment tool Kallisto [@bray2016near] was chosen above other quantification methods as it is performing equally good but faster. For this study, a subset of 172 tumors (samples) with high-risk disease were selected, forming two groups: the MYCN amplified ($n_1$ = 91) and MYCN non-amplified ($n_2$ = 81) tumours as used in [@Assefa2018]. Sometimes we refer this dataset to us the Zhang data or the Zhang neuroblastoma data. A subset of this dataset (5000 randomly selected genes) is available with the SPsimSeq package for illustration purpose only.

```{r loadData, eval=TRUE}  
 # load the Zhang bulk RNA-seq data (availabl with the package) 
 data("zhang.data.sub") 
  # filter genes with sufficient expression (important step to avoid bugs) 
 zhang.counts <- zhang.data.sub$counts 
 MYCN.status  <- zhang.data.sub$MYCN.status #The grouping variable
```

This dataset is now used as a template for semiparametric data generation. We simulate only a single data (n.sim = 1) with the following properties:
 - 3000 genes ( n.genes = 3000) 
 - 172 samples (tot.samples = 172) -- equal to the source data
 - the samples are equally divided into 2 groups each with 90 samples 
   (group.config = c(0.5, 0.5)) -- almost equal to the source data
 - all samples are from a single batch (batch.config = 1)
 - we add 10% DE genes (pDE = 0.1) 
 - the DE genes have a log-fold-change of at least 0.5 in 
   the source data (lfc.thrld = 0.5)
 - we do not model the zeroes separately, they are the part of density 
    estimation (model.zero.prob = FALSE)

```{r simulateData} 
 set.seed(6452) #Set seed for reproducibility
 # simulate data
 sim.data.bulk <- SPsimSeq(n.sim = 1, s.data = zhang.counts,
                          group = MYCN.status, n.genes = 3000, batch.config = 1,
                          group.config = c(0.5, 0.5), tot.samples = ncol(zhang.counts), 
                          pDE = 0.1, lfc.thrld = 0.5, result.format = "list", return.details = TRUE)
```
 
Next, we explore the data we just generated.

```{r dataExploration}
 sim.data.bulk1 <- sim.data.bulk$sim.data.list[[1]]
 head(sim.data.bulk1$counts[, seq_len(5)])  # count data
 head(sim.data.bulk1$colData)        # sample info
 head(sim.data.bulk1$rowData)        # gene info
```

Since we set _return.details_ = TRUE, we have access to all density estimates, which can be extracted with the _evaluateDensities_ function.

```{r evaluateDensities}
geneDens = evaluateDensities(sim.data.bulk, newData = rownames(zhang.counts)[1])
#This returns for every sample, the midpoints (mids) and associated densities (gy)
```

Next we compare the data generated with SPsimSeq with the original data properties to show that they are realistic and close to the real data.
 
```{r comparison, warning=FALSE, fig.width=8, fig.height=4}
# compare the distributions of the mean expressions, variability, 
# and fraction of zero counts per gene
library(LSD) # for generating heatmap plots
# normalize counts for comparison  
Y0.log.cpm <- log2(edgeR::cpm(zhang.counts)+1)
Y1.log.cpm <- log2(edgeR::cpm(sim.data.bulk1$counts)+1)
Y0.log.cpm <- Y0.log.cpm[rowMeans(Y0.log.cpm>0)>=0.1, ]
Y1.log.cpm <- Y1.log.cpm[rowMeans(Y1.log.cpm>0)>=0.1, ]
rowVars <- function(X){apply(X, 1, var, na.rm=TRUE)}
rowCVs <- function(X){apply(X, 1, function(x) sd(x, na.rm=TRUE)/mean(x, na.rm=TRUE))}
par(mfrow=c(1, 3))
boxplot(list(real.data=log(colSums(zhang.counts)), 
             simulated.data=log(sim.data.bulk1$colData$sim.Lib.Size)), 
        main="library size") 
boxplot(list(real.data=rowMeans(Y0.log.cpm), 
             simulated.data=rowMeans(Y1.log.cpm)), 
        main="mean expression of genes") 
boxplot(list(real.data=rowVars(Y0.log.cpm), 
             simulated.data=rowVars(Y1.log.cpm)), 
        main="variance of gene expressions") 
```

The library sizes are identical since they were not modelled (see _variable.lib.size_ argument in ?SPsimSeq). Next, we look at mean-variance trends

```{r meanvariancetrend}
# compare the relationship between the mean and variability
par(mfrow=c(1,3), mar=c(4,4,4,1))
heatscatter(rowMeans(Y0.log.cpm), rowCVs(Y0.log.cpm), ylim=c(0, 6), xlim=c(0, 16),
            colpal="bl2gr2rd", main="real data", xlab="mean log2-CPM", 
            ylab="coefficients of variation", cexplot=0.5, alpha = 60, cex.lab=1.25)
heatscatter(rowMeans(Y1.log.cpm), rowCVs(Y1.log.cpm), ylim=c(0, 6), xlim=c(0, 16),
     main="SPsimSeq", xlab="mean log2-CPM", ylab="coefficients of variation", 
     cexplot=0.5, alpha = 60, colpal="bl2gr2rd", cex.lab=1.25)

n.gride <- 1000
min.g   <- seq(0, 20, length.out = n.gride+1)[-n.gride]
max.g   <- seq(0, 20, length.out = n.gride+1)[-1] 
mid.g   <- (min.g+max.g)/2
f.real  <- vapply(seq_len(n.gride), FUN.VALUE = double(1), function(r){
  x <- Y0.log.cpm[rowMeans(Y0.log.cpm)<=max.g[r] & rowMeans(Y0.log.cpm)>min.g[r],]
  y <- ifelse(!is.null(dim(x)), mean(rowCVs(x)), mean(sd(x)/mean(x))) 
  y
})
f.SPsim <- vapply(seq_len(n.gride), FUN.VALUE = double(1), function(r){
  x <- Y1.log.cpm[rowMeans(Y1.log.cpm)<=max.g[r] & rowMeans(Y1.log.cpm)>min.g[r],]
  y <- ifelse(!is.null(dim(x)), mean(rowCVs(x)), mean(sd(x)/mean(x))) 
  y
})

sm1 <- loess(I(f.SPsim-f.real)~mid.g) 
plot(mid.g, f.SPsim-f.real, xlim=c(0, 14), col="lightskyblue", pch=20, cex.lab=1.25,
     cex.main=1.4, main="SPsimSeq - real data", ylab="difference", xlab="mean log2-CPM")
lines(mid.g,predict(sm1, newdata = mid.g), col="blue", lwd=3) 
```

We emulated the correlation networks found in the real data (see _genewiseCor_ argument). Now we check if the correlation patterns in the synthetic data resemble those in the real data.

```{r correlation}
# compare the correlation between genes and samples 
cor.mat.Y0 <- cor(t(Y0.log.cpm))
cor.mat.Y1 <- cor(t(Y1.log.cpm)) 
cor.vec.Y0 <- cor.mat.Y0[upper.tri(cor.mat.Y0)]
cor.vec.Y1 <- cor.mat.Y1[upper.tri(cor.mat.Y1)] 
par(mfrow=c(1,3), mar=c(4,4,3.5,1))
hist(cor.vec.Y0, nclass = 30, probability = TRUE, 
     border="gray", col="steelblue1", main="real data", xlab="Genewise correlations", 
     ylim=c(0, 3.5), xlim=c(-1, 1), cex.lab=1.25)
hist(cor.vec.Y1, nclass = 30, probability = TRUE, border="gray",
     col="steelblue1",  main="SPsimSeq", xlab="Genewise correlations",
     ylim=c(0, 3.5), xlim=c(-1, 1), cex.lab=1.25)
plot(seq(-1, 1, 0.1), seq(-1, 1, 0.1), type="n", xlab="quantile (real data)", 
     ylab="quantile (simulated data)",  main="correlation quantile-quantile plot")
abline(0, 1, col="gray")
points(quantile(cor.vec.Y0, seq(0, 1, 0.001)), quantile(cor.vec.Y1, seq(0, 1, 0.001)), 
       col="blue", pch=20, cex=1.5, cex.lab=1.25)  
```

## Example 2: simulating single-cell RNA-seq (containing read-counts)
**Neuroblastoma NGP cells scRNA-seq data (NGP data)** retrieved from [@Verboom430090] (GEO accession GSE119984): This dataset is generated for a cellular perturbation experiment on the C1 instrument (SMARTer protocol) [@Verboom430090]. This total RNA-seq dataset contains 83 NGP neuroblastoma cells, of which 31 were treated with 8$\mu$M of nutlin-3 and the other 52 cells were treated with vehicle (controls). In the subsequent sections, this dataset is referred to us the NGP single-cell RNA-seq data.

We simulate only a single scRNA-seq data (n.sim = 1) with the following property
  - 4000 genes (n.genes = 4000) 
  - 100 cells (tot.samples = 100) 
  - the cells are equally divided into 2 groups each with 50 cells 
    (group.config = c(0.5, 0.5))
  - all cells are from a single batch (batch.config = 1)
  - we add 10% DE genes (pDE = 0.1) 
  - the DE genes have a log-fold-change of at least 0.5
  - we model the zeroes separately (model.zero.prob = TRUE)
  - the ouput will be in SingleCellExperiment class object (result.format = "SCE")

```{r scRNA, eval=TRUE, warning=FALSE, fig.width=8, fig.height=4}
library(SingleCellExperiment)
# load the NGP nutlin data (availabl with the package, processed with SMARTer/C1 protocol, and contains read-counts)
  data("scNGP.data")
  set.seed(654321)
 # simulate data (we simulate here only a single data, n.sim = 1)
 sim.data.sc <- SPsimSeq(n.sim = 1, s.data = scNGP.data,
                         group = scNGP.data$characteristics..treatment, 
                         n.genes = 4000, batch.config = 1,
                         group.config = c(0.5, 0.5), tot.samples = 100, 
                         pDE = 0.1, lfc.thrld = 0.5, model.zero.prob = TRUE,
                         result.format = "SCE")
```

Take a quick peek at the data.
 
```{r scRNAhead}
 sim.data.sc1 <- sim.data.sc[[1]]
 class(sim.data.sc1)
 head(counts(sim.data.sc1)[, seq_len(5)])
 colData(sim.data.sc1)
 rowData(sim.data.sc1)
```
 
Look at basic data properties.

```{r basicSCrna} 
# normalize counts for comparison  
Y0.log.cpm  <- log2(edgeR::cpm(counts(scNGP.data))+1)
Y1.log.cpm  <- log2(edgeR::cpm(counts(sim.data.sc1))+1)
Y0.log.cpm  <- Y0.log.cpm[rowMeans(Y0.log.cpm>0)>=0.1, ]
Y1.log.cpm  <- Y1.log.cpm[rowMeans(Y1.log.cpm>0)>=0.1, ]
rowVars <- function(X){apply(X, 1, var, na.rm=TRUE)}
rowCVs <- function(X){apply(X, 1, function(x) sd(x, na.rm=TRUE)/mean(x, na.rm=TRUE))}
rowZeroFrac <- function(X){apply(X, 1, function(x) mean(x==0, na.rm=TRUE))}
par(mfrow=c(1, 3))
boxplot(list(real.data=colSums(counts(scNGP.data)), 
             simulated.data=colData(sim.data.sc1)$sim.Lib.Size), 
        main="library size") 
boxplot(list(real.data=rowMeans(Y0.log.cpm), 
             simulated.data=rowMeans(Y1.log.cpm)), 
        main="mean expression of genes") 
boxplot(list(real.data=rowVars(Y0.log.cpm), 
             simulated.data=rowVars(Y1.log.cpm)), 
        main="variance of gene expressions")
```

Compare mean-variance distributions

```{r meanVarSCrna}
# compare the relationship between the mean and variability  
par(mfrow=c(1,3), mar=c(4,4,4,1))
heatscatter(rowMeans(Y0.log.cpm), rowCVs(Y0.log.cpm), ylim=c(0, 6), xlim=c(0, 16),
            colpal="bl2gr2rd", main="real data", xlab="mean log2-CPM", 
            ylab="coefficients of variation", cexplot=0.5, alpha = 60, cex.lab=1.25)
heatscatter(rowMeans(Y1.log.cpm), rowCVs(Y1.log.cpm), ylim=c(0, 6), xlim=c(0, 16),
     main="SPsimSeq", xlab="mean log2-CPM", ylab="coefficients of variation", 
     cexplot=0.5, alpha = 60, colpal="bl2gr2rd", cex.lab=1.25)
n.gride <- 1000
min.g   <- seq(0, 20, length.out = n.gride+1)[-n.gride]
max.g   <- seq(0, 20, length.out = n.gride+1)[-1] 
mid.g   <- (min.g+max.g)/2
f.real  <- vapply(seq_len(n.gride), FUN.VALUE = double(1), function(r){
  x <- Y0.log.cpm[rowMeans(Y0.log.cpm)<=max.g[r] & rowMeans(Y0.log.cpm)>min.g[r],]
  y <- ifelse(!is.null(dim(x)), mean(rowCVs(x)), mean(sd(x)/mean(x))) 
  y
})
f.SPsim <- vapply(seq_len(n.gride), FUN.VALUE = double(1), function(r){
  x <- Y1.log.cpm[rowMeans(Y1.log.cpm)<=max.g[r] & rowMeans(Y1.log.cpm)>min.g[r],]
  y <- ifelse(!is.null(dim(x)), mean(rowCVs(x)), mean(sd(x)/mean(x))) 
  y
})

sm1 <- loess(I(f.SPsim-f.real)~mid.g) 
plot(mid.g, f.SPsim-f.real, xlim=c(0, 14), col="lightskyblue", pch=20, cex.lab=1.25,
     cex.main=1.4, main="SPsimSeq - real data", ylab="difference", xlab="mean log2-CPM")
lines(mid.g,predict(sm1, newdata = mid.g), col="blue", lwd=3)
```

Means and zeroes

```{r scRNAmeanzeroes}
# compare the relationship between the  mean and fraction of zeros
par(mfrow=c(1,3), mar=c(4,4,4,1))
heatscatter(rowMeans(Y0.log.cpm), rowZeroFrac(Y0.log.cpm), ylim=c(0, 1), 
            xlim=c(0, 16), colpal="bl2gr2rd", main="real data", xlab="mean log2-CPM", 
            ylab="fraction of zero counts", cexplot=0.5, alpha = 60, cex.lab=1.25)
heatscatter(rowMeans(Y1.log.cpm), rowZeroFrac(Y1.log.cpm), ylim=c(0, 1), 
            xlim=c(0, 16), main="SPsimSeq", xlab="mean log2-CPM", 
            ylab="fraction of zero counts",  cexplot=0.5, alpha = 60,
            colpal="bl2gr2rd", cex.lab=1.25)
n.gride <- 1000
min.g   <- seq(0, 20, length.out = n.gride+1)[-n.gride]
max.g   <- seq(0, 20, length.out = n.gride+1)[-1] 
mid.g   <- (min.g+max.g)/2
f.real  <- vapply(seq_len(n.gride), FUN.VALUE = double(1), function(r){
  x <- Y0.log.cpm[rowMeans(Y0.log.cpm)<=max.g[r] & rowMeans(Y0.log.cpm)>min.g[r],]
  y <- ifelse(!is.null(dim(x)), mean(rowZeroFrac(x)), mean(x==0)) 
  y
})
f.SPsim <- vapply(seq_len(n.gride), FUN.VALUE = double(1), function(r){
  x <- Y1.log.cpm[rowMeans(Y1.log.cpm)<=max.g[r] & rowMeans(Y1.log.cpm)>min.g[r],]
  y <- ifelse(!is.null(dim(x)), mean(rowZeroFrac(x)), mean(x==0)) 
  y
})
sm1 <- loess(I(f.SPsim-f.real)~mid.g) 
plot(mid.g, f.SPsim-f.real, xlim=c(0, 14), col="lightskyblue", pch=20, cex.lab=1.25,
     cex.main=1.4, main="SPsimSeq - real data", ylab="difference", xlab="mean log2-CPM")
lines(mid.g,predict(sm1, newdata = mid.g), col="blue", lwd=3)
```

Also here we look at the correlation networks

```{r corSCrna}
# compare the correlation between genes and samples
Y0.log.cpm2 <- Y0.log.cpm[rowMeans(Y0.log.cpm>0)>0.25, ]
Y1.log.cpm2 <- Y1.log.cpm[rowMeans(Y1.log.cpm>0)>0.25, ]

cor.mat.Y0 <- cor(t(Y0.log.cpm2))
cor.mat.Y1 <- cor(t(Y1.log.cpm2)) 

cor.vec.Y0 <- cor.mat.Y0[upper.tri(cor.mat.Y0)]
cor.vec.Y1 <- cor.mat.Y1[upper.tri(cor.mat.Y1)] 

par(mfrow=c(1,3), mar=c(4,4,3.5,1))
hist(cor.vec.Y0, nclass = 30, probability = TRUE, 
     border="gray", col="steelblue1", main="real data", xlab="pairwise correlation between genes", 
     ylim=c(0, 3.5), xlim=c(-1, 1), cex.lab=1.25)
hist(cor.vec.Y1, nclass = 30, probability = TRUE, border="gray",
     col="steelblue1",  main="SPsimSeq", xlab="pairwise correlation between genes",
     ylim=c(0, 3.5), xlim=c(-1, 1), cex.lab=1.25)

plot(seq(-1, 1, 0.1), seq(-1, 1, 0.1), type="n", xlab="quantile (real data)", 
     ylab="quantile (simulated data)",  main="correlation quantile-quantile plot")
abline(0, 1, col="gray")
points(quantile(cor.vec.Y0, seq(0, 1, 0.001)), quantile(cor.vec.Y1, seq(0, 1, 0.001)), 
       col="blue", pch=20, cex=1.5, cex.lab=1.25)  
```

```{r}
sessionInfo()
```

# References

\printbibliography

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selectGenes.R
\name{selectGenes}
\alias{selectGenes}
\title{Sample genes from candidate genes}
\usage{
selectGenes(pDE, exprmt.design, n.genes, null.genes0, nonnull.genes0)
}
\arguments{
\item{pDE}{fraction of genes to be made DE}

\item{exprmt.design}{the experiment design}

\item{n.genes}{the total number of genes required}

\item{null.genes0, nonnull.genes0}{Candidate null and non-null genes}
}
\value{
a vector of selected genes
}
\description{
Sample genes from candidate genes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitPoisGlm.R
\name{fitPoisGlm}
\alias{fitPoisGlm}
\title{Fast fit Poisson regression}
\usage{
fitPoisGlm(Ny, x, degree, offset)
}
\arguments{
\item{Ny}{vector of counts}

\item{x}{regressor}

\item{degree}{degree of the polynomial}

\item{offset}{offset}
}
\value{
see glm.fit
}
\description{
Fast fit Poisson regression
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genCopula.R
\name{genCopula}
\alias{genCopula}
\title{Generate a copula instance}
\usage{
genCopula(corMats, exprmt.design)
}
\arguments{
\item{corMats}{List of correlation matrices}

\item{exprmt.design}{Number of batches, and batch vector}
}
\value{
a list of copula instances
}
\description{
Generate a copula instance
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/samZeroID.R
\name{samZeroID}
\alias{samZeroID}
\title{Return ID for observations to be set to zero}
\usage{
samZeroID(fracZero.logit.list, logLL, gene)
}
\arguments{
\item{fracZero.logit.list}{The estimated zero model}

\item{logLL}{the logged library sizes}

\item{gene}{the gene name}
}
\value{
A boolean, should a zero be introduced or not?
}
\description{
Return ID for observations to be set to zero
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evaluateDensities.R
\name{evaluateDensities}
\alias{evaluateDensities}
\title{Evaluate the densities in the estimated SPsimSeq object}
\usage{
evaluateDensities(SPobj, newData = names(SPobj$detailed.results$densList))
}
\arguments{
\item{SPobj}{The SPsimSeq object, with details retained}

\item{newData}{A character vector of gene names}
}
\value{
a list of estimated densities, breaks and midpoints, one for every 
gene in newData
}
\description{
Evaluate the densities in the estimated SPsimSeq object
}
\examples{
data("zhang.data.sub")
# filter genes with sufficient expression (important step to avoid bugs)
zhang.counts <- zhang.data.sub$counts
MYCN.status  <- zhang.data.sub$MYCN.status
# simulate data
sim.data.bulk <- SPsimSeq(n.sim = 1, s.data = zhang.counts,
                          group = MYCN.status, n.genes = 2000, batch.config = 1,
                          group.config = c(0.5, 0.5), tot.samples = 20,
                          pDE = 0.1, lfc.thrld = 0.5, result.format = "list",
                          return.details = TRUE)
outDens = evaluateDensities(sim.data.bulk)
select.genes <- sample(names(outDens), 4)
select.sample = sample(
seq_along(sim.data.bulk$detailed.results$exprmt.design$sub.groups), 1)
par(mfrow=c(2, 2))
for(i in select.genes){
     plot(outDens[[i]][[select.sample]]$mids, outDens[[i]][[select.sample]]$gy, type = "l", 
     xlab = "Outcome", ylab = "Density", main = paste("Gene", i))
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracZeroLogitModel.R
\name{fracZeroLogitModel}
\alias{fracZeroLogitModel}
\title{Extract data and iterate over batches to estimate zero probability models}
\usage{
fracZeroLogitModel(s.data, batch, cpm.data, n.mean.class, minFracZeroes)
}
\arguments{
\item{s.data, cpm.data}{raw and transformed data}

\item{batch}{the batch vector}

\item{n.mean.class}{see zeroProbModel}

\item{minFracZeroes}{minimum fraction of zeroes before zero-inflation is applied}
}
\value{
a list of binomial regression parameters
}
\description{
Extract data and iterate over batches to estimate zero probability models
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitLLmodel.R
\name{fitLLmodel}
\alias{fitLLmodel}
\title{Fit log linear model for each gene}
\usage{
fitLLmodel(yy, mu.hat, sig.hat, n)
}
\arguments{
\item{yy}{a list object contating a result from obtCount() function for a single gene}

\item{mu.hat, sig.hat}{Carrier density estimators}

\item{n}{number of observations}
}
\value{
a list object containing the fitted log linear model and carrier density
}
\description{
Fit log linear model for each gene
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/obtCorMatsBatch.R
\name{obtCorMatsBatch}
\alias{obtCorMatsBatch}
\title{A function to obtain copulas or uniform random variables}
\usage{
obtCorMatsBatch(cpm.data, batch)
}
\arguments{
\item{cpm.data}{the transformed data matrix}

\item{batch}{the batch indicators}
}
\value{
The estimated correlation matrices per batch
}
\description{
A function to obtain copulas or uniform random variables
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculateCPM.R
\name{calculateCPM}
\alias{calculateCPM}
\title{Calculates counts per millions of reads, possibly with log-transform}
\usage{
calculateCPM(X, const.mult, prior.count)
}
\arguments{
\item{X}{raw data matrix}

\item{const.mult}{a constant to multiply with}

\item{prior.count}{prior count to be added to the zeroes}
}
\value{
a normalized data matrix
}
\description{
Calculates counts per millions of reads, possibly with log-transform
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SPsimSeq-package.R
\docType{package}
\name{SPsimSeq-package}
\alias{SPsimSeq-package}
\title{SPsimSeq package}
\description{
SPsimSeq uses a specially designed exponential family for density estimation 
to constructs the distribution of gene expression levels from a given real RNA 
sequencing data (single-cell or bulk), and subsequently, simulates a new 
dataset from the estimated marginal distributions using Gaussian-copulas to 
retain the dependence between genes. It allows simulation of multiple 
groups and batches with any required sample size and library size.
}
\references{
\itemize{
\item Alemu Takele Assefa, Jo Vandesompele, Olivier Thas. (2020). SPsimSeq: semi-parametric simulation of bulk and single cell RNA sequencing data, \emph{Bioinformatics}, , btaa105, https://doi.org/10.1093/bioinformatics/btaa105
}
}
\author{
Alemu Takele Assefa \email{alemutak@hotmail.com}

Stijn Hawinkel \email{stijnhawinkel@hotmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NGPscData.R
\docType{data}
\name{scNGP.data}
\alias{scNGP.data}
\title{Neuroblastoma NGP cells single-cell RNA-seq.}
\format{
A SingleCellExperiment object
}
\source{
GEO accession GSE119984
}
\usage{
scNGP.data
}
\description{
It was retrieved from [1] (GEO accession GSE119984): This dataset is 
 generated for a cellular perturbation experiment on the C1 instrument 
 (SMARTer protocol) [1]. This total RNA-seq dataset contains 
 83 NGP neuroblastoma cells, of which 31 were treated with 
 nutlin-3 and the other 52 cells were treated with vehicle (controls).
}
\examples{
data("scNGP.data")
scNGP.data
}
\references{
1 - Verboom, K., Everaert, C., Bolduc, N., Livak, K. J., Yigit, N., Rombaut, D., ... & Speleman, F. (2019). SMARTer single cell total RNA sequencing. Nucleic Acids Research, 47(16), e93-e93.

\describe{
  \item{SingleCellExperiment}{counts + gene info + cell infro} 
}
}
\keyword{dataset}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/buildXmat.R
\name{buildXmat}
\alias{buildXmat}
\title{An auxialiary function to quickly construct the polyomial matrix, 
using Horner's rule}
\usage{
buildXmat(x, nc)
}
\arguments{
\item{x}{The base}

\item{nc}{the number of columns}
}
\value{
A matrix with increasing powers of x in the columns
}
\description{
An auxialiary function to quickly construct the polyomial matrix, 
using Horner's rule
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chooseCandGenes.R
\name{chooseCandGenes}
\alias{chooseCandGenes}
\title{Select candidate genes}
\usage{
chooseCandGenes(
  cpm.data,
  group,
  lfc.thrld,
  llStat.thrld,
  t.thrld,
  w = w,
  max.frac.zeror.diff = Inf,
  pDE,
  n.genes,
  prior.count
)
}
\arguments{
\item{cpm.data}{logCPM transformed matrix (if log.CPM.transform=FALSE, then it is the source gene expression data)}

\item{group}{a grouping factor}

\item{lfc.thrld}{a positive numeric value for the minimum absolute log-fold-change for selecting candidate DE genes in the source data (when group is not NULL and pDE>0)}

\item{llStat.thrld}{a positive numeric value for the minimum squared test statistics from the log-linear model to select candidate DE genes in the source data (when group is not NULL and  pDE>0)
containing X as a covariate to select DE genes}

\item{t.thrld}{a positive numeric value for the minimum absolute t-test statistic for the log-fold-changes of genes for selecting candidate DE genes in the source data (when group is not NULL and  pDE>0)}

\item{w}{a numeric value between 0 and 1. The number of classes to construct the probability distribution will be round(w*n), where n is the total number of samples/cells in a particular batch of the source data}

\item{max.frac.zeror.diff}{a numeric value >=0 indicating the maximum absolute
difference in the fraction of zero counts between the groups for DE genes.}

\item{pDE}{fraction of DE genes}

\item{n.genes}{total number of genes}

\item{prior.count}{a positive constant to be added to the CPM before log transformation, to avoid log(0). The default is 1.}
}
\value{
a list object contating a set of candidate null and non-null genes and additional results
}
\description{
This function can be used to independently select candidate genes from a given real RNA-srq data (bulk/single)
for the SPsimSeq simulation. It chooses genes with various chracteristics, such as log-fold-change 
above a certain thereshold.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zhangData.R
\docType{data}
\name{zhang.data.sub}
\alias{zhang.data.sub}
\title{Neuroblastoma bulk RNA-seq data retrieved from Zhang et (2015).}
\format{
A list object
}
\source{
\url{GEO accession GSE49711}
}
\usage{
data(zhang.data.sub)
}
\description{
The data contains 498 neuroblastoma tumors. In short, unstranded 
poly(A)+ RNA sequencing was performed on the HiSeq 2000 instrument (Illumina). 
Paired-end reads with a length of 100 nucleotides were obtained. To quantify 
the full transcriptome, raw fastq files were processed with Kallisto v0.42.4 
(index build with GRCh38-Ensembl v85). The pseudo-alignment tool Kallisto 
was chosen above other quantification methods as it is performing equally 
good but faster. For this study, a subset of 172 tumors (samples) with 
high-risk disease were selected, forming two groups: the MYCN amplified 
($n_1$ = 91) and MYCN non-amplified ($n_2$ = 81) tumours. Sometimes we refer 
this dataset to us the Zhang data or the Zhang neuroblastoma data. In this
package, a subset of 5000 genes (randomly selected) are made available for illustration
purpose only.
}
\examples{
data("zhang.data.sub")
str(zhang.data.sub)
}
\references{
1. Zhang W, Yu Y, Hertwig F, Thierry-Mieg J, Zhang W, Thierry-Mieg D, Wang J, Furlanello C, Devanarayan V, Cheng J, et al. Comparison of RNA-seq and microarray-based models for clinical endpoint prediction. Genome Biol. 2015;16(133) https://doi.org/10.1186/s13059-015-0694-1
2. Assefa, A. T., De Paepe, K., Everaert, C., Mestdagh, P., Thas, O., & Vandesompele, J. (2018). Differential gene expression analysis tools exhibit substandard performance for long non-coding RNA-sequencing data. GENOME BIOLOGY, 19.
\describe{
  \item{counts}{gene counts}
  \item{group}{MYCN (0 for MYCN non-amplified and 1 for MYCN amplified)}
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SPsimPerGene.R
\name{SPsimPerGene}
\alias{SPsimPerGene}
\title{A function that generates the simulated data for a single gene}
\usage{
SPsimPerGene(
  cumDens,
  exprmt.design,
  sel.genes.ii,
  log.CPM.transform,
  prior.count,
  LL,
  copSam,
  model.zero.prob,
  fracZero.logit.list,
  const.mult
)
}
\arguments{
\item{cumDens}{cumulative density}

\item{exprmt.design}{the experiment design}

\item{sel.genes.ii}{selected gene}

\item{log.CPM.transform}{a boolean, is log-CPM transform required?}

\item{prior.count}{the prior count}

\item{LL}{the library sizes}

\item{copSam}{the generated copula}

\item{model.zero.prob}{a boolean, should the zeroes be modelled separately}

\item{fracZero.logit.list}{The zero model}

\item{const.mult}{a large constant for the CPM transform, normally 1e6}
}
\value{
Simulated cpm values
}
\description{
A function that generates the simulated data for a single gene
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constructDens.R
\name{constructDens}
\alias{constructDens}
\title{Construct the cumulative density}
\usage{
constructDens(densList.ii, exprmt.design, DE.ind.ii, returnDens = FALSE)
}
\arguments{
\item{densList.ii}{the estimated density parameters}

\item{exprmt.design}{experiment configuration}

\item{DE.ind.ii}{a boolean, is the gene to be DE?}

\item{returnDens}{A boolean, should densities rather than cumulative 
densities be returned?}
}
\value{
The cumulative density
}
\description{
Construct the cumulative density
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genLibSizes.R
\name{genLibSizes}
\alias{genLibSizes}
\title{Generate library sizes from log-normal}
\usage{
genLibSizes(fit.ln, exprmt.design)
}
\arguments{
\item{fit.ln}{the library size model}

\item{exprmt.design}{the design}
}
\value{
The generated libray sizes per batch and group
}
\description{
Generate library sizes from log-normal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneParmEst.R
\name{geneParmEst}
\alias{geneParmEst}
\title{Gene level param estimates for density estimation}
\usage{
geneParmEst(
  cpm.data.i,
  batch,
  group,
  prior.count = prior.count,
  de.ind,
  model.zero.prob,
  w
)
}
\arguments{
\item{cpm.data.i}{full vector of genewise observation}

\item{batch, group}{batch and grouping vectors}

\item{prior.count}{the prior count for the cpm transofrm}

\item{de.ind}{a boolean, is the gene to be DE?}

\item{model.zero.prob}{a boolean, should zero-density be modelled?}

\item{w}{weight}
}
\value{
list of density estimates
}
\description{
Gene level param estimates for density estimation
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estLibSizeDistr.R
\name{estLibSizeDistr}
\alias{estLibSizeDistr}
\title{Estimate log-normal distribution for the library sizes}
\usage{
estLibSizeDistr(LS, batch)
}
\arguments{
\item{LS}{observed library sizes}

\item{batch}{batches}
}
\value{
Estimated log-normal parameter library sizes
}
\description{
Estimate log-normal distribution for the library sizes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matchCopula.R
\name{matchCopula}
\alias{matchCopula}
\title{Match copulas to estimated SP distribution}
\usage{
matchCopula(cumDens, exprmt.design, copSam, sel.genes.ii)
}
\arguments{
\item{cumDens}{The cumulative densities evaluated}

\item{exprmt.design}{the design}

\item{copSam}{the sampled copula}

\item{sel.genes.ii}{the gene}
}
\value{
the outcome values as a vector
}
\description{
Match copulas to estimated SP distribution
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zeroProbModel.R
\name{zeroProbModel}
\alias{zeroProbModel}
\title{Predict zero probability using logistic rgression}
\usage{
zeroProbModel(cpm.data, logL, zeroMat, n.mean.class)
}
\arguments{
\item{cpm.data}{log CPM matrix}

\item{logL}{log library size of the source data}

\item{zeroMat}{the matrix of zero indicators}

\item{n.mean.class}{a fraction of the number of genes
for the number of groups to be created for the mean log CPM of genes}
}
\value{
The coefficients of the estimated logistic regression
}
\description{
Predict zero probability using logistic rgression
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/configExperiment.R
\name{configExperiment}
\alias{configExperiment}
\title{Configure experiment}
\usage{
configExperiment(batch.config, group.config, tot.samples, batch, group)
}
\arguments{
\item{batch.config}{a numerical vector for the marginal fraction of samples in each batch. 
The number of batches to be simulated is equal to the size of the vector.
All values must sum to 1.}

\item{group.config}{a numerical vector for the marginal fraction of samples in each group.
The number of groups to be simulated is equal to the size of the vector. All values must sum to 1.}

\item{tot.samples}{total number of samples to be simulated.}

\item{batch, group}{batch and grouping vectors}
}
\value{
a list object contating the number of groups and batches to be simukated, 
and the experiment configurartion
}
\description{
Configure experiment
}
\examples{
batch = sample(LETTERS[1:3], 20, replace = TRUE)
group = sample(1:3, 20, replace = TRUE)
#---- a design with a total of 10 samples/cells from 1 batch and 1 group
configExperiment(batch.config=1, group.config=1, tot.samples=10, 
batch = batch, group = group)

#---- a design with a total of 20 samples/cells from 1 group and 2 batchs with 
# batch 1 has 15 samples/cells and batch 2 has 5
configExperiment(batch.config = c(15/20, 5/20), group.config = 1, 
tot.samples = 20, batch = batch, group = group)

#---- a design with a total of 20 samples/cells from 1 batch and 2 groups with 
# group 1 has 10 samples/cells and batch 2 has 10
configExperiment(batch.config=1, group.config=c(0.5, 0.5), tot.samples=20, 
batch = batch, group = group)

#---- a design with a total of 30 samples/cells from 2 groups with group 1 has 15 samples 
# and group 2 has 15, and  three batchs with batch 1,2, and 3 have 5, 10, and 15 samples/cells, 
# respectively.
configExperiment(batch.config = c(5/30, 10/30, 15/30), group.config = c(0.5, 0.5),
 tot.samples = 30, batch = batch, group = group)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneParmEst.R
\name{parmEstDensVec}
\alias{parmEstDensVec}
\title{Density estimation on a single vector}
\usage{
parmEstDensVec(
  Y0,
  model.zero.prob,
  min.val,
  w,
  prev.min.val = 0.25,
  min.count.nonnull = 3
)
}
\arguments{
\item{Y0}{the vector of observations}

\item{model.zero.prob, min.val, w}{see geneParmEst()}

\item{prev.min.val}{minimum prevalence of minimum values}

\item{min.count.nonnull}{minimum count for estimation}
}
\value{
density estimates
}
\description{
Density estimation on a single vector
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/samZeroID.R
\name{expit}
\alias{expit}
\title{Evaluate the expit function}
\usage{
expit(x)
}
\arguments{
\item{x}{the argument}
}
\value{
the expit of the argument
}
\description{
Evaluate the expit function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/obtCount.R
\name{obtCount}
\alias{obtCount}
\title{Calculates height and mid points of a distribution}
\usage{
obtCount(Y, w)
}
\arguments{
\item{Y}{a vector of gene expression data for a particular gene (in log CPM)}

\item{w}{a numeric value between 0 and 1 or NULL refering the number of classes to be created}
}
\value{
a list object contating class breaks, mid points and counts
}
\description{
Calculates height and mid points of a distribution
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extractMat.R
\name{extractMat}
\alias{extractMat}
\alias{extractMat,SingleCellExperiment-method}
\alias{extractMat,matrix-method}
\alias{extractMat,data.frame-method}
\alias{extractMat,phyloseq-method}
\title{A function with S4 dispatching to extract the count matrix}
\usage{
extractMat(Y, ...)

\S4method{extractMat}{SingleCellExperiment}(Y, ...)

\S4method{extractMat}{matrix}(Y, ...)

\S4method{extractMat}{data.frame}(Y, ...)

\S4method{extractMat}{phyloseq}(Y, ...)
}
\arguments{
\item{Y}{a matrix, data frame, phyloseq object or SingleCellExperiment}

\item{...}{additional arguments, currently ignored}
}
\value{
A data matrix with samples in the columns and genes in the rows
}
\description{
A function with S4 dispatching to extract the count matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepareSPsimOutputs.R
\name{prepareSPsimOutputs}
\alias{prepareSPsimOutputs}
\title{A function to prepare outputs}
\usage{
prepareSPsimOutputs(sim.dat, exprmt.design, DE.ind, result.format, LL)
}
\arguments{
\item{sim.dat}{The simulated data}

\item{exprmt.design}{the design}

\item{DE.ind}{the differential abundance indicator}

\item{result.format}{the desired output format}

\item{LL}{simulated library sizes}
}
\value{
the data in the desired format
}
\description{
A function to prepare outputs
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SPsimSeq.R
\name{SPsimSeq}
\alias{SPsimSeq}
\title{A function to simulate bulk or single cell RNA sequencing data}
\usage{
SPsimSeq(
  n.sim = 1,
  s.data,
  batch = rep(1, ncol(s.data)),
  group = rep(1, ncol(s.data)),
  n.genes = 1000,
  batch.config = 1,
  group.config = 1,
  pDE = 0.1,
  cand.DE.genes = NULL,
  lfc.thrld = 0.5,
  t.thrld = 2.5,
  llStat.thrld = 5,
  tot.samples = ncol(s.data),
  model.zero.prob = FALSE,
  genewiseCor = TRUE,
  log.CPM.transform = TRUE,
  lib.size.params = NULL,
  variable.lib.size = FALSE,
  w = NULL,
  result.format = "SCE",
  return.details = FALSE,
  verbose = TRUE,
  prior.count = 1,
  const.mult = 1e+06,
  n.mean.class = 0.2,
  minFracZeroes = 0.25
)
}
\arguments{
\item{n.sim}{an integer for the number of simulations to be generated}

\item{s.data}{a real source dataset (a SingleCellExperiment class or a matrix/data.frame of counts with genes in rows and samples in columns)}

\item{batch}{NULL or a vector containing batch indicators for each sample/cell in the source data}

\item{group}{NULL or a vector containing group indicators for each sample/cell in the source data}

\item{n.genes}{a numeric value for the total number of genes to be simulated}

\item{batch.config}{a numerical vector containing fractions for the composition of samples/cells per batch. The fractions must sum to 1. The number of batches to be simulated is equal to the size of the vector. (Example, batch.config=c(0.6, 0.4)   means simulate 2 batches with 60\% of the simulated samples/cells in batch 1 and the rest 40\% in the second batch. Another example,  batch.config=c(0.3, 0.35, 0.25) means simulate 3 batches with the first, second and third batches contain 30\%, 35\% and 25\% of the samples/cells, respectively).}

\item{group.config}{a numerical vector containing fractions for the composition of samples/cells per group. Similar definition to 'batch.config'. The number of groups to be simulated is equal to the size of the vector. The fractions must sum to 1.}

\item{pDE}{a numeric value between 0 and 1 indicating the desired fraction of DE genes in the simulated data}

\item{cand.DE.genes}{a list object contatining canidiate null and non-null (DE/predictor) genes.
If NULL (the default), an internal function determines candidate genes based on log-fold-change and other statistics.
The user can also pass a list of canidate null and non-null genes (they must be disjoint). In particular, the list should contain
two character vectors (for the name of the features/genes in the source data) with names 'null.genes' and 'nonnull.genes'.
For example, cand.DE.genes=list(null.genes=c('A', 'B'), nonnull.genes=c('C', 'D')).}

\item{lfc.thrld}{a positive numeric value for the minimum absolute log-fold-change for selecting
candidate DE genes in the source data (when group is not NULL, pDE>0 and cand.DE.genes is NULL)}

\item{t.thrld}{a positive numeric value for the minimum absolute t-test statistic for the
log-fold-changes of genes for selecting candidate DE genes in the source data (when group
is not NULL, pDE>0 and cand.DE.genes is NULL)}

\item{llStat.thrld}{a positive numeric value for the minimum squared test statistics from
the log-linear model to select candidate DE genes in the source data (when group is not NULL,
pDE>0 and cand.DE.genes is NULL)}

\item{tot.samples}{a numerical value for the total number of samples/cells to be simulated.}

\item{model.zero.prob}{a logical value whether to model the zero expression probability separately (suitable for simulating of single-cell RNA-seq data or zero-inflated data)}

\item{genewiseCor}{a logical value, if TRUE (default) the simulation will retain the gene-to-gene correlation structure of the source data using Gausian-copulas . Note that if it is TRUE, the program will be slow or it may fail for a limited memory size.}

\item{log.CPM.transform}{a logical value. If TRUE, the source data will be transformed into log-(CPM+const) before estimating the probability distributions}

\item{lib.size.params}{NULL or a named numerical vector containing parameters for simulating library sizes from log-normal distribution. If lib.size.params =NULL (default), 
then the package will fit a log-normal distribution for the library sizes in the source data to simulate new library sizes. 
If the user would like to specify the parameters of the log-normal distribution for the desired library sizes, 
then the log-mean and log-SD params of rlnorm() functions can be passed using this argument. 
Example, lib.size.params = c(meanlog=10, sdlog=0.2). See also ?rlnorm.}

\item{variable.lib.size}{a logical value. If FALSE (default), then the expected library sizes are simulated once and remains the same for every replication (if n.sim>1).}

\item{w}{see ?hist}

\item{result.format}{a character value for the type of format for the output. Choice can be 'SCE' for SingleCellExperiment class or "list" for a list object that contains the simulated count, column information and row information.}

\item{return.details}{a logical value. If TRUE, detailed results including estimated parameters and densities will be returned}

\item{verbose}{a logical value, if TRUE a message about the status of the simulation will be printed on the console}

\item{prior.count}{a positive constant to be added to the CPM before log transformation, to avoid log(0). The default is 1.}

\item{const.mult}{A constant by which the count are scaled. Usually 1e6 to get CPM}

\item{n.mean.class}{a fraction of the number of genes
for the number of groups to be created for the mean log CPM of genes}

\item{minFracZeroes}{minimum fraction of zeroes before a zero inflation model
is fitted}
}
\value{
a list of SingleCellExperiment/list objects each containing simulated counts (not normalized), smple/cell level information in colData, and gene/feature level information in rowData.
}
\description{
This function simulates (bulk/single cell) RNA-seq dataset from semi-parametrically estimated distributions of gene expression levels in a given real source RNA-seq dataset
}
\details{
This function uses a specially designed exponential family for density estimation
to constructs the distribution of gene expression levels from a given real gene expression data
(e.g. single-cell or bulk sequencing data), and subsequently, simulates a new
from the estimated distributions.#'
For simulation of single-cell RNA-seq data (or any zero inflated gene expression data), the programm
involves an additional step to explicitly account for the high abundance of zero counts (if required).
This step models the probability of zero counts as a function the mean
expression of the gene and the library size of the cell (both in log scale) to add excess zeros.
This can be done by using \emph{model.zero.prob=TRUE}. Note that,
for extremly large size data, it is recomended to use a random sample of cells to
reduce computation time. To enable this, add the argument \emph{subset.data=TRUE} and you
can specify the number of cells to be used using \emph{n.samples} argument.
For example \emph{n.samples=400}.
Given known groups of samples/cells in the source data, DGE is simulated by independently
sampling data from distributions constructed for each group seprately. In particular, this procedure is
applied on a set of genes with absolute log-fold-change in the source data more than a given threshold (\emph{lfc.thrld}).
Moreover, when  the source dataset involves samples/cells processed in different batches, our
simulation procedure incorporates this batch effect in the simulated data, if required.
Different experimental designs can be simulated using the group and batch configuration arguments to
simulate biologica/experimental conditions and batchs, respectively.
Also, it is important to filter the source data so that genes with suffient expression will be used to
estimate the probability distributions.
}
\examples{
#----------------------------------------------------------------
# Example 1: simulating bulk RNA-seq

# load the Zhang bulk RNA-seq data (availabl with the package)
data("zhang.data.sub")

zhang.counts <- zhang.data.sub$counts
MYCN.status  <- zhang.data.sub$MYCN.status

# We simulate only a single data (n.sim = 1) with the following property
# - 1000 genes ( n.genes = 1000)
# - 40 samples (tot.samples = 40)
# - the samples are equally divided into 2 groups each with 90 samples
#   (group.config = c(0.5, 0.5))
# - all samples are from a single batch (batch = NULL, batch.config = 1)
# - we add 10\% DE genes (pDE = 0.1)
# - the DE genes have a log-fold-change of at least 0.5 in
#   the source data (lfc.thrld = 0.5)
# - we do not model the zeroes separately, they are the part of density
#    estimation (model.zero.prob = FALSE)

# simulate data
set.seed(6452)
sim.data.bulk <- SPsimSeq(n.sim = 1, s.data = zhang.counts,
                          group = MYCN.status, n.genes = 1000, batch.config = 1,
                          group.config = c(0.5, 0.5), tot.samples = 40,
                          pDE = 0.1, lfc.thrld = 0.5, result.format = "list")

head(sim.data.bulk$counts[[1]][, seq_len(5)])  # count data
head(sim.data.bulk$colData)        # sample info
head(sim.data.bulk$rowData)        # gene info

#----------------------------------------------------------------
# Example 2: simulating single cell RNA-seq from a single batch (read-counts)
# we simulate only a single scRNA-seq data (n.sim = 1) with the following property
# - 2000 genes (n.genes = 2000)
# - 100 cells (tot.samples = 100)
# - the cells are equally divided into 2 groups each with 50 cells
#   (group.config = c(0.5, 0.5))
# - all cells are from a single batch (batch = NULL, batch.config = 1)
# - we add 10\% DE genes (pDE = 0.1)
# - the DE genes have a log-fold-change of at least 0.5
# - we model the zeroes separately (model.zero.prob = TRUE)
# - the ouput will be in SingleCellExperiment class object (result.format = "SCE")

library(SingleCellExperiment)

# load the NGP nutlin data (availabl with the package, processed with
# SMARTer/C1 protocol, and contains read-counts)
data("scNGP.data")

# filter genes with sufficient expression (important step to avoid bugs)
treatment <- ifelse(scNGP.data$characteristics..treatment=="nutlin",2,1)

set.seed(654321)

# simulate data (we simulate here only a single data, n.sim = 1)
sim.data.sc <- SPsimSeq(n.sim = 1, s.data = scNGP.data, group = treatment,
 n.genes = 2000, batch.config = 1, group.config = c(0.5, 0.5), 
 tot.samples = 100, pDE = 0.1, lfc.thrld = 0.5, model.zero.prob = TRUE,
                    result.format = "SCE")

sim.data.sc1 <- sim.data.sc[[1]]
class(sim.data.sc1)
head(counts(sim.data.sc1)[, seq_len(5)])
colData(sim.data.sc1)
rowData(sim.data.sc1)

}
\references{
\itemize{
\item Assefa, A. T., Vandesompele, J., & Thas, O. (2020). SPsimSeq: semi-parametric simulation of bulk and single cell RNA sequencing data. \emph{Bioinformatics}, doi: https://doi.org/10.1093/bioinformatics/btaa105.
\item Efron, B., & Tibshirani, R. (1996). Using specially designed exponential families for density estimation. \emph{The Annals of Statistics}, 24(6), 2431-2461.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkInputValidity.R
\name{checkInputValidity}
\alias{checkInputValidity}
\title{Check for data validity}
\usage{
checkInputValidity(
  s.data,
  group,
  batch,
  group.config,
  batch.config,
  w,
  log.CPM.transform,
  prior.count,
  pDE,
  lib.size.params,
  llStat.thrld,
  result.format
)
}
\arguments{
\item{s.data, group, batch, group.config, batch.config, w, log.CPM.transform, prior.count, pDE, lib.size.params, llStat.thrld, result.format}{see ?SPsimSeq}
}
\value{
Throws errors where neede, otherwise returns invisible
}
\description{
Check for data validity
}
